from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ConditionInput(TcBaseObj):
    """
    Holds the name of the condition to be evaluated along with the set of input parameter business object values.
    
    :var conditionName: This is the name of the condition to be evaluated.
    :var conditionSignature: This is set of condition parameters (tag_t) required by the specified condition name.
    """
    conditionName: str = ''
    conditionSignature: List[BusinessObject] = ()


@dataclass
class ConditionOutput(TcBaseObj):
    """
    Holds the results of a condition evaluation along with any exit code that was captured during the condition
    evaluation.
    
    :var result: This is the result of the rules engine evaluation (True or False).
    :var exitCode: This is the exit code (zero for success or non zero indicating an error) captured during the
    condition evaluation.
    """
    result: bool = False
    exitCode: int = 0


@dataclass
class EvaluateConditionsResponse(TcBaseObj):
    """
    Holds the response for the 'evaluateConditions' operation.
    
    :var outputs: This is a set of condition evaluation results from the rules engine execution.
    :var serviceData: This contains the status of the operation.
    """
    outputs: List[ConditionOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Multisite._2011_06.ImportExportAsync import RemoteExportInfo
from tcsoa.base import TcService


class ImportExportAsyncService(TcService):

    @classmethod
    def remoteExport(cls, reInfos: List[RemoteExportInfo]) -> None:
        """
        The 'remoteExport' operation is used to export given business objects from the source site to the target site
        asynchronously. The export options supplied in the 'RemoteExportInfo' structure influences the business object
        export.  
        E.g. In case user supplies Include All revision as TRUE, from source site all versions of the dataset will be
        exported to target site.  
        
        When option(s) are not specified in the RemoteExportInfo structure, by default user session preferences at the
        exporting site are considered and default values are mentioned in the ImportExportOptionsInfo structure. 
        
        This operation is integrated with Multi-Site RPC framework and is callable only in Global Workflow based
        Multi-Site environment. Consequently no other clients can invoke this operation other than Global Workflow
        clients. This operation does not return any values.
        
        
        Use cases:
        - In the multisite federation, user can share the business objects among the participating sites. The client
        shares business object(s) using the remote import or remote export functionality. 
        - To remote export the business object, using the Rich Client (RAC), the source site user creates an item and
        exports it to the target site. At the target site, user performs local search with the item id. The local
        search should provide the item, which confirms the successful remote export. 
        - Also business object remote export can be achieved using the command line utility data_share f=send. 
        
        """
        return cls.execute_soa_method(
            method_name='remoteExport',
            library='Internal-Multisite',
            service_date='2011_06',
            service_name='ImportExportAsync',
            params={'reInfos': reInfos},
            response_cls=None,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanFile, VariantRule
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class EffectivityInfo(TcBaseObj):
    """
    This structure contains  information about the effectivity configuration.
    Note: If effectivityUnitNo is not to be considered then -1 has to be passed as the default value. Value  0 is
    considered as  a valid effectivity unit number.
    
    :var effectivityUnitNo: Effective unit number.
    :var effectivityDate: Effective date configuration information.
    :var endItem: Effective end item.
    """
    effectivityUnitNo: int = 0
    effectivityDate: datetime = None
    endItem: BusinessObject = None


@dataclass
class GetIndexedProductsResponse(TcBaseObj):
    """
    This structure contains the list of product and configuration for which the product structure information is
    indexed.
    
    :var indexedProductAndConfigInfos: A list of product and configuration for which the product structure information
    are indexed.
    :var serviceData: ServiceData in which the partial errors are communicated to the client.
    """
    indexedProductAndConfigInfos: List[IndexedProductOutput] = ()
    serviceData: ServiceData = None


@dataclass
class OccsGroupedByProperty(TcBaseObj):
    """
    A structure that contains a list of PFUIDs, grouped by the property names and values.
    
    :var internalPropertyName: The internal name of the property.
    :var groupedPfuidsMap: A map (string, list of strings) containing a list of 'propertyGroupID' for each PFUID. For
    multi-valued properties, a single PFUID may be associated with multiple 'propertyGroupIDs'.
    :var unmatchedPfuidList: List of unmatched pfuids.
    """
    internalPropertyName: str = ''
    groupedPfuidsMap: PfuidToPropertyGroupIdMap = None
    unmatchedPfuidList: List[str] = ()


@dataclass
class OccsGroupedByPropertyResponse(TcBaseObj):
    """
    A structure that contains a list of PFUIDs, grouped by the property names and values.
    
    :var groupedOccList: List of 'OccGroupedByProperty' information.
    :var serviceData: The service data through which errors are communicated to the client.
    """
    groupedOccList: List[OccsGroupedByProperty] = ()
    serviceData: ServiceData = None


@dataclass
class ProductAndConfigInfo(TcBaseObj):
    """
    This structure contains product and the associated configuration for which the product structure information is
    indexed.
    
    :var product: The product which is indexed. It could be an ItemRevision.
    :var revisionRule: The RevisionRule configuration applied on the product structure.
    :var effectivityInfos: A list of effectivity information for which the product structure was configured.
    :var variantRuleInfos: A list of VariantRule applied on the product structure.
    """
    product: BusinessObject = None
    revisionRule: BusinessObject = None
    effectivityInfos: List[EffectivityInfo] = ()
    variantRuleInfos: List[VariantRuleInfo] = ()


@dataclass
class ProductAndConfigInfoInput(TcBaseObj):
    """
    This structure contains product and the associated configuration for which the product structure information file
    has to be retrieved. In case the client always needs a full structure then the deltaIdentifier can be passed as
    empty in which case the complete structure file along with any delta product structure files will be returned.
    The product information for which the product structure information has to be retrieved can be provided using the
    recipeObject or the productConfigInfo.recipeObject if provided would take precedence over productConfigInfo.
    
    
    :var clientId: A clientId passed by the client in order to map the input to the corresponding output in the
    response.
    :var deltaIdentifier: Delta identifier to identify mark point from which the next set of delta need to be retrieved.
    :var recipeObject: The BusinessObject that contains the information about the product and the configuration. For
    example it could be Awb0ProductContextInfo or VisStructureContext.
    :var productConfigInfo: The product and the configuration information for which the product structure is being
    retrieved.
    """
    clientId: str = ''
    deltaIdentifier: str = ''
    recipeObject: BusinessObject = None
    productConfigInfo: ProductAndConfigInfo = None


@dataclass
class ProductStructureFileInfo(TcBaseObj):
    """
    This structure contains the read ticket to a file containing the complete product structure information at some
    point in time and a set of read tickets to files containing the delta product structure information. The order of
    deltaStructureFileReadTickets is the order in which the delta files has to be merged with the complete file to make
    the product structure information current.
    
    :var clientId: A clientId passed by the client in order to map the input to the corresponding output in the
    response.
    :var fullStructureFileReadTicket: Read ticket to file that contains the full product structure at some point in
    time.
    :var deltaStructureFileReadTickets: Read tickets to file containing delta product structure information since the
    complete product structure file was generated.
    :var nextDeltaIdentifierToken: A delta token that need to be used for identifying  the next set of delta product
    structure file since this request.
    """
    clientId: str = ''
    fullStructureFileReadTicket: str = ''
    deltaStructureFileReadTickets: List[str] = ()
    nextDeltaIdentifierToken: str = ''


@dataclass
class ProductStructureIdInfo(TcBaseObj):
    """
    This structure contains the id of the product structure and the configuration information that need to be applied
    to the product structure.
    
    :var clientId: A clientId passed by the client in order to map the input to the corresponding output in the
    response.
    :var productStrutureIdentifier: The identifier to identify the product structure information. This identifier
    corresponds to the product id in the product structure(mmp file).
    :var productAndConfig: The configuration that need to be applied on the  product structure.
    """
    clientId: str = ''
    productStrutureIdentifier: str = ''
    productAndConfig: ProductAndConfigInfo = None


@dataclass
class ProductStructureIdInput(TcBaseObj):
    """
    This structure contains product and the associated configuration for which the product structure information id is
    to be determined. The configuration information can either be passed in a detailed format or as recipe object.
    
    :var clientId: A clientId passed by the client in order to map the input to the corresponding output in the
    response.
    :var recipeObject: The object that contains the configuration recipe. It could be something like
    Awb0ProductContextInfo or VisStructureContext.
    :var productAndConfig: The product and configuration information for which the structure id has to be determined.
    """
    clientId: str = ''
    recipeObject: BusinessObject = None
    productAndConfig: ProductAndConfigInfo = None


@dataclass
class PropertyGroupingValue(TcBaseObj):
    """
    A structure containing start and end values for a specific property. The end value is used for range comparisons if
    populated.
    
    :var propertyGroupID: Unique Identifier used by client to identify the group. The id will allow to associate the
    input to the output. This allows multiple types or ranges share the same color.
    
    :var startValue: String representation of the value for the property. For ranges, this is the start value for the
    range. If the client code is dealing with specific value types (int, double, etc.) the client code can use the
    appropriate client APIs to convert values to a string representation e.g Property::toFloatString,
    Property::toIntString, Property::toDateString, etc. On the server side, they can be converted back to the
    appropriate value types using the corresponding APIs e.g Property::parseFloat, Property::parseInt,
    Property::parseDate, etc.
    :var endValue: String representation of the end value for the property. This is optional and is populated only for
    ranges. It represents the end value of the range. See the startValue description for how the client  and server
    code can convert from and to the specific value types.
    """
    propertyGroupID: str = ''
    startValue: str = ''
    endValue: str = ''


@dataclass
class GetProductStructureIdResponse(TcBaseObj):
    """
    This structure contains the information about the product structure information identifier which can be used to
    determine the product structure information from the catalogue of product structure information maintained at the
    client. The configuration that need to be applied on that product structure information is also returned via this
    response.
    
    :var psIdInfos: A list of product and configuration for which the product structure information are indexed. 
    :var serviceData: ServiceData by which the partial errors are communicated to the client.
    """
    psIdInfos: List[ProductStructureIdInfo] = ()
    serviceData: ServiceData = None


@dataclass
class UpdateCollectionInput(TcBaseObj):
    """
    This structure contains  information about the file that need to be associated to the Awv0MMPDeltaCollection
    identified by the deltaCollectionIdentifier.
    
    :var deltaCollectionIdentifier: The string used to uniquely identify the Awv0MMPDeltaCollection instance. This is
    an uid of of Awb0BOMIndexAdminData instance in cases of Active Content Experience(ACE). In other non ACE usecases
    this is the string by which the Awb0ProductStructureProvider identifies the Awv0MMPDeltaCollection instance. 
    :var productStructureFile: The file that need to be associated to the Awv0MMPDeltaCollection dataset.
    :var isFull: If true, the file being supplied represents the complete product structure information. If false the
    file being supplied represents the delta product structure information.
    """
    deltaCollectionIdentifier: str = ''
    productStructureFile: ImanFile = None
    isFull: bool = False


@dataclass
class VariantRuleInfo(TcBaseObj):
    """
    This structure contains VariantRule object that was applied for configuration and the object to which the
    VariantRule is associated through GRM relation.
    
    :var variantRule: The VariantRule  applied for configuration.
    :var variantRuleOwningObject: The object to which the VariantRule is associated via ImanRelation.
    """
    variantRule: VariantRule = None
    variantRuleOwningObject: BusinessObject = None


@dataclass
class GetStructureFilesResponse(TcBaseObj):
    """
    This structure contains the information about the files containing product structure information that make up a
    complete product structure.
    
    :var productStructureFileInfos: A list of ProductStructureFileInfo containing the read ticket to the complete and
    or delta product structure files.
    :var serviceData: ServiceData by which the partial errors are communicated to the client.
    """
    productStructureFileInfos: List[ProductStructureFileInfo] = ()
    serviceData: ServiceData = None


@dataclass
class GroupOccsByPropertyInput(TcBaseObj):
    """
    A structure containing an internal property name, a list of 'PFUIDs 'that are to be grouped based upon
    'PropertyGroupingValue'.
    
    :var internalPropertyName: The internal name of the property based on whose values the occurrences are to be
    grouped.
    :var pfuidList: List of pfuid by which the product structure line can be identified in ACE Index Bom.
    :var propertyValues: List of 'PropertyGroupingValue' information corresponding to the property.
    """
    internalPropertyName: str = ''
    pfuidList: List[str] = ()
    propertyValues: List[PropertyGroupingValue] = ()


@dataclass
class IndexedProductOutput(TcBaseObj):
    """
    This structure contains the product, associated configuration information along with an identifier that is used to
    indentify this product and configuration in a catalogue of structure files.
    
    :var productConfigInfo: The product and the configuration information.
    :var productStructureIdentifier: An identifier to identify this product and configuration. The structure files for
    this product and configuration can be identified by this identifier.
    """
    productConfigInfo: ProductAndConfigInfo = None
    productStructureIdentifier: str = ''


"""
A map containing a list of 'propertyGroupID' for each PFUID. For multi-valued properties, a single PFUID may be associated with multiple 'propertyGroupIDs'. The id will allow to associate the input to the output. This allows multiple types or ranges share the same color.

"""
PfuidToPropertyGroupIdMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Query._2012_10.Finder import SearchFilterField, SearchSortCriteria
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ObjectPropertyGroupInput(TcBaseObj):
    """
    A structure containing an internal property name, a list of PropertyGroupingValue  objects and a list of Business
    Objects.
    
    :var internalPropertyName: The internal name for the property used for grouping
    :var propertyValues: List of start and end property values for the internalPropertyName.
    :var objectList: List of business objects to be grouped. This is typically a subset of search results that need to
    be grouped.
    """
    internalPropertyName: str = ''
    propertyValues: List[PropertyGroupingValue] = ()
    objectList: List[BusinessObject] = ()


@dataclass
class ObjectsGroupedByProperty(TcBaseObj):
    """
    A structure containing an internal property name and a map of objects to the property group id. It also contains a
    list of unmatched objects which dont match any group.
    
    :var internalPropertyName: The internal name of the property.
    :var groupedObjectsMap: Map  (business object, list of strings) of selected business object and its associated
    grouping names.
    :var unmatchedObjectList: List of unmatched business objects.
    """
    internalPropertyName: str = ''
    groupedObjectsMap: ObjectPropertyGroupingMap = None
    unmatchedObjectList: List[BusinessObject] = ()


@dataclass
class ObjectsGroupedByPropertyResponse(TcBaseObj):
    """
    A structure containing list of ObjectPropertyGrouping objects and service data.
    
    :var groupedObjectsList: List of ObjectPropertyGrouping objects.
    :var serviceData: The service data.
    """
    groupedObjectsList: List[ObjectsGroupedByProperty] = ()
    serviceData: ServiceData = None


@dataclass
class PropertyGroupingValue(TcBaseObj):
    """
    A structure containing start and end values for a specific property.  The end value is used for range comparisons
    if populated
    
    :var propertyGroupID: Unique Identifier used by client to identify the group.
    :var startValue: String representation of the value for the property. For ranges, this is the start value for the
    range. If the client code is dealing with specific value types (int, double, etc.) the client code can use the
    appropriate client APIs to convert values to a string representation e.g Property::toFloatString,
    Property::toIntString, Property::toDateString, etc. On the server side, they can be converted back to the
    appropriate value types using the corresponding APIs e.g Property::parseFloat, Property::parseInt,
    Property::parseDate, etc.
    :var endValue: String representation of the end value for the property. This is optional and is populated only for
    ranges. It represents the end value of the range.See the startValue description for how the client and server code
    can convert from and to the specific value types.
    """
    propertyGroupID: str = ''
    startValue: str = ''
    endValue: str = ''


@dataclass
class SearchFilter2(TcBaseObj):
    """
    A structure representing a string, date or numeric type of search filter.
    
    :var searchFilterType: The type of search filter. Valid values are "StringFilter", "DateFilter", "NumericFilter"
    :var stringValue: The value for a string filter. This field is applicable only if the "searchFilterType" field is
    set to "StringFilter".
    :var stringDisplayValue: The display value for a string filter. This field is applicable only if the
    "searchFilterType" field is set to "StringFilter".
    :var startDateValue: The starting value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter";.
    :var endDateValue: The ending value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: A flag that indicates if the filter was previously selected and used to filter the results. 
    :var startEndRange: The 'gap' used to generate the start and end values
    """
    searchFilterType: str = ''
    stringValue: str = ''
    stringDisplayValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    startEndRange: str = ''


@dataclass
class SearchInput2(TcBaseObj):
    """
    A structure containing input search criteria.
    
    :var providerName: The name of the search provider.  This is the RuntimeBusinessObject type name.
    :var searchCriteria: The criteria used to perform search (string/string). For example, for object set search, the
    search criteria are parentUid and object set source string.
    :var startIndex: The start index to return the search results
    :var maxToReturn: The maximum number of search results to return. If value is  zero, no search results are
    returned; however, other information such as total results found and filters details are returned.
    :var maxToLoad: The maximum number of search results to load
    :var searchFilterMap: The map (string,list of SearchFilter )containing the list of search filters for each search
    filter field.
    :var searchSortCriteria: The criteria to use to sort the results.
    :var searchFilterFieldSortType: The sorting type to use to order the search filter categories in the response. The
    acceptable values are: "Alphabetical","Priority".
    :var attributesToInflate: A list of attributes to inflate.
    :var internalPropertyName: The internal name of the property used for grouping.
    """
    providerName: str = ''
    searchCriteria: StringMap2 = None
    startIndex: int = 0
    maxToReturn: int = 0
    maxToLoad: int = 0
    searchFilterMap: SearchFilterMap2 = None
    searchSortCriteria: List[SearchSortCriteria] = ()
    searchFilterFieldSortType: str = ''
    attributesToInflate: List[str] = ()
    internalPropertyName: str = ''


@dataclass
class SearchResponse2(TcBaseObj):
    """
    A service response structure containing search results
    
    :var searchResults: The list of business objects obtained after performing a search. 
    :var totalFound: The total number of business objects found
    :var totalLoaded: The total number of business objects loaded
    :var searchFilterMap: The map (string,list of SearchFilter )containing the list of search filters for each search
    filter field
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    :var serviceData: The service data object.
    :var objectsGroupedByProperty: The structure containing an internal property name and a map of objects to the
    property group id. It also contains a list of unmatched objects which don't match any group.
    """
    searchResults: List[BusinessObject] = ()
    totalFound: int = 0
    totalLoaded: int = 0
    searchFilterMap: SearchFilterMap2 = None
    searchFilterCategories: List[SearchFilterField] = ()
    defaultFilterFieldDisplayCount: int = 0
    serviceData: ServiceData = None
    objectsGroupedByProperty: ObjectsGroupedByProperty = None


"""
A map containing a list of propertyGroupID for each BusinessObject. For multi-valued properties, a single business object may be associated with multiple property group IDs.
"""
ObjectPropertyGroupingMap = Dict[BusinessObject, List[str]]


"""
A map containing a list of search filters for each search filter field.
"""
SearchFilterMap2 = Dict[str, List[SearchFilter2]]


"""
Map from string to string
"""
StringMap2 = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetValidCriteriaResponse(TcBaseObj):
    """
    Returns the vector of map of criteria names (This vector will be the same size as the inputScope vector). In
    addition any partial errors are returned in the serviceData member.
    
    :var criteriamap: The map of criteria name(string) and localized names (vector). Currently both the key and value
    are the same. The size of this vector matches the size of the inputScope vector.
    :var serviceData: serviceData to return any partial errors
    """
    criteriamap: List[CriteriaNamesMap] = ()
    serviceData: ServiceData = None


@dataclass
class ValidCriteriaInput(TcBaseObj):
    """
    Provides a set of input values to get the valid criteria
    
    :var sourcescope: Selected source tags to find out the valid critera for accountability check.
    :var targetscope: Selected target tags to find the valid criteria for accountability check.
    """
    sourcescope: List[BusinessObject] = ()
    targetscope: List[BusinessObject] = ()


"""
Map for criteria names. Key is the real name and value is localized name.
"""
CriteriaNamesMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.StructureManagement._2008_05.StructureSearch import OccurrenceNoteExpression, SearchBySizeExpression, FormAttributeExpression, ProximityExpression, BoxZoneExpression, InClassExpression, SavedQueryExpression, AttributeExpression
from typing import Dict, List
from tcsoa.gen.StructureManagement._2010_04.StructureSearch import PlaneZoneExpression
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.BusinessObjects import Item


@dataclass
class SearchExpressionSet(TcBaseObj):
    """
    SearchExpression data structure specifies the full set of search expressions that are to be used to perform a
    single structure search. Each one of the expressions provided are combined together using a logical 'AND' between
    them.
    
    :var itemAndRevisionAttributeExpressions: A collection of Item and ItemRevision attribute search expressions
    :var occurrenceNoteExpressions: A collection of Occurrence Note attribute search expressions
    :var returnScopedSubTreesHit: Specifies whether the search result should return only the subset of 'scopeBomLines'
    given as input as part of the 'SearchScope'
    :var executeVOOFilter: If true, Valid Overlays Only (VOO) filter is applied prior to returning results from the
    server.
    :var executeRemoteSearch: Specifies whether the search should include results from the remote site in response.
    :var remoteSiteID: The remote site id to search from. By default, if the site id is not specified, the search will
    use the site id that is configured by the admin in Teamcenter.
    :var formAttributeExpressions: A collection of Form attribute search expressions
    :var proximitySearchExpressions: A collection of spatial proximity search expressions
    :var boxZoneExpressions: A collection of spatial box zone expressions
    :var planeZoneExpressions: A collection of spatial plane zone search expressions
    :var savedQueryExpressions: A collection of saved query search expressions
    :var inClassQueryExpressions: A collection of inclass attribute search expressions
    :var sizeSearchExpression: A collection of spatial size search expressions
    :var doTrushapeRefinement: Specifies whether to perform trushape (use actual geometry rather using bounding box
    covering min and max points) refinement on the spatial search
    """
    itemAndRevisionAttributeExpressions: List[AttributeExpression] = ()
    occurrenceNoteExpressions: List[OccurrenceNoteExpression] = ()
    returnScopedSubTreesHit: bool = False
    executeVOOFilter: bool = False
    executeRemoteSearch: bool = False
    remoteSiteID: str = ''
    formAttributeExpressions: List[FormAttributeExpression] = ()
    proximitySearchExpressions: List[ProximityExpression] = ()
    boxZoneExpressions: List[BoxZoneExpression] = ()
    planeZoneExpressions: List[PlaneZoneExpression] = ()
    savedQueryExpressions: List[SavedQueryExpression] = ()
    inClassQueryExpressions: List[InClassExpression] = ()
    sizeSearchExpression: SearchBySizeExpression = None
    doTrushapeRefinement: bool = False


@dataclass
class BoundingBoxInfo(TcBaseObj):
    """
    Contains the six Coordinates of the Bbox
    
    :var xmin: Minimum 'x' coordinate value for bounding box.
    :var ymin: Minimum 'y' coordinate value for bounding box.
    :var zmin: Minimum 'z' coordinate value for bounding box.
    :var xmax: Maximum 'x' coordinate value for bounding box.
    :var ymax: Maximum 'y' coordinate value for bounding box.
    :var zmax: Maximum 'z' coordinate value for bounding box.
    """
    xmin: float = 0.0
    ymin: float = 0.0
    zmin: float = 0.0
    xmax: float = 0.0
    ymax: float = 0.0
    zmax: float = 0.0


@dataclass
class BoundingBoxInfoResponse(TcBaseObj):
    """
    A map of Item passed in the input list to the 'BoundingBoxInfo' object, for the corresponding Item.
    
    :var boundingBoxes: Map of(Item/ 'BoundingBoxInfo') for each input Item and corresponding bounding box coordinates.
    :var serviceData: An object of 'ServiceData' which contains any exceptions if occurred during getting bounding box.
    """
    boundingBoxes: BoundingBoxs = None
    serviceData: ServiceData = None


"""
Item and corresponding bbox coordinates are mapped
"""
BoundingBoxs = Dict[Item, BoundingBoxInfo]



from tcsoa.gen.Manufacturing._2012_02.services import DataManagementService as imp0
from tcsoa.gen.Manufacturing._2014_06.services import DataManagementService as imp1
from tcsoa.gen.Manufacturing._2008_06.services import TimeManagementService as imp2
from tcsoa.gen.Manufacturing._2012_09.services import DataManagementService as imp3
from tcsoa.gen.Manufacturing._2009_10.services import ModelService as imp4
from tcsoa.gen.Manufacturing._2010_09.services import TimeManagementService as imp5
from tcsoa.gen.Manufacturing._2014_06.services import ResourceManagementService as imp6
from tcsoa.gen.Manufacturing._2013_05.services import IPAManagementService as imp7
from tcsoa.gen.Manufacturing._2009_06.services import StructureManagementService as imp8
from tcsoa.gen.Manufacturing._2011_06.services import DataManagementService as imp9
from tcsoa.gen.Manufacturing._2009_10.services import MFGPropertyCollectorService as imp10
from tcsoa.gen.Manufacturing._2013_12.services import ModelService as imp11
from tcsoa.gen.Manufacturing._2009_10.services import StructureManagementService as imp12
from tcsoa.gen.Manufacturing._2018_11.services import StructureManagementService as imp13
from tcsoa.gen.Manufacturing._2013_05.services import DataManagementService as imp14
from tcsoa.gen.Manufacturing._2015_10.services import StructureManagementService as imp15
from tcsoa.gen.Manufacturing._2009_10.services import DataManagementService as imp16
from tcsoa.gen.Manufacturing._2014_12.services import StructureSearchService as imp17
from tcsoa.gen.Manufacturing._2008_06.services import DataManagementService as imp18
from tcsoa.gen.Manufacturing._2012_02.services import IPAManagementService as imp19
from tcsoa.gen.Manufacturing._2008_12.services import IPAManagementService as imp20
from tcsoa.gen.Manufacturing._2012_09.services import ValidationService as imp21
from tcsoa.gen.Manufacturing._2016_03.services import ImportExportService as imp22
from tcsoa.gen.Manufacturing._2013_05.services import StructureManagementService as imp23
from tcsoa.gen.Manufacturing._2014_12.services import IPAManagementService as imp24
from tcsoa.gen.Manufacturing._2008_06.services import CoreService as imp25
from tcsoa.gen.Manufacturing._2010_09.services import CoreService as imp26
from tcsoa.gen.Manufacturing._2013_05.services import CoreService as imp27
from tcsoa.gen.Manufacturing._2013_05.services import StructureSearchService as imp28
from tcsoa.gen.Manufacturing._2017_05.services import ValidationService as imp29
from tcsoa.gen.Manufacturing._2012_02.services import ModelService as imp30
from tcsoa.gen.Manufacturing._2017_11.services import DataManagementService as imp31
from tcsoa.gen.Manufacturing._2009_10.services import ModelDefinitionsService as imp32
from tcsoa.gen.Manufacturing._2014_12.services import ValidationService as imp33
from tcsoa.gen.Manufacturing._2018_06.services import DataManagementService as imp34
from tcsoa.gen.Manufacturing._2012_02.services import ConstraintsService as imp35
from tcsoa.gen.Manufacturing._2011_06.services import StructureManagementService as imp36
from tcsoa.gen.Manufacturing._2013_12.services import ResourceManagementService as imp37
from tcsoa.gen.Manufacturing._2013_05.services import TimeWayPlanService as imp38
from tcsoa.gen.Manufacturing._2017_05.services import ImportExportService as imp39
from tcsoa.gen.Manufacturing._2013_05.services import ImportExportService as imp40
from tcsoa.gen.Manufacturing._2015_10.services import ImportExportService as imp41
from tcsoa.gen.Manufacturing._2010_09.services import ImportExportService as imp42
from tcsoa.gen.Manufacturing._2015_03.services import StructureManagementService as imp43
from tcsoa.gen.Manufacturing._2009_10.services import StructureSearchService as imp44
from tcsoa.gen.Manufacturing._2019_06.services import DataManagementService as imp45
from tcsoa.gen.Manufacturing._2014_06.services import StructureSearchService as imp46
from tcsoa.gen.Manufacturing._2014_12.services import ModelService as imp47
from tcsoa.base import TcService


class DataManagementService(TcService):
    addAssociatedContexts = imp0.addAssociatedContexts
    addOrRemoveAssociatedContexts = imp1.addOrRemoveAssociatedContexts
    applyConfigObjects = imp3.applyConfigObjects
    associateAndAllocateByPreview = imp0.associateAndAllocateByPreview
    automaticAllocatePreview = imp0.automaticAllocatePreview
    automaticAssociateAndAllocate = imp0.automaticAssociateAndAllocate
    cloneAssemblyAndProcessObjects = imp1.cloneAssemblyAndProcessObjects
    closeContexts = imp9.closeContexts
    closeViews = imp9.closeViews
    connectObjects = imp0.connectObjects
    createAttachments = imp14.createAttachments
    createObjects = imp16.createObjects
    createOrUpdateConfigObjects = imp3.createOrUpdateConfigObjects
    createOrUpdateMEActivityFolders = imp18.createOrUpdateMEActivityFolders
    createOrUpdateMENXObjects = imp18.createOrUpdateMENXObjects
    disconnectFromOrigin = imp0.disconnectFromOrigin
    disconnectObjects = imp16.disconnectObjects
    establishOriginLink = imp1.establishOriginLink
    getAssociatedContexts = imp0.getAssociatedContexts
    getConnectorInfo = imp31.getConnectorInfo
    getOccurrenceKinematicsInformation = imp34.getOccurrenceKinematicsInformation
    getPhysicalAttachmentsInScope = imp31.getPhysicalAttachmentsInScope
    getProcessResourceRelatedInfo = imp1.getProcessResourceRelatedInfo
    openContexts = imp9.openContexts
    openViews = imp9.openViews
    publishSelectionFromStudyToSource = imp45.publishSelectionFromStudyToSource
    removePhysicalAttachementRelation = imp31.removePhysicalAttachementRelation
    setAttributes = imp14.setAttributes
    setConnectorInfo = imp31.setConnectorInfo
    setOccurrenceKinematicsInformation = imp34.setOccurrenceKinematicsInformation
    setPhysicalAttachementsInScope = imp31.setPhysicalAttachementsInScope
    syncSelectionInStudyWithSource = imp45.syncSelectionInStudyWithSource
    syncStudyAndSource = imp14.syncStudyAndSource


class TimeManagementService(TcService):
    allocatedTimeRollUp = imp2.allocatedTimeRollUp
    calculateCriticalPathEx = imp5.calculateCriticalPathEx
    getActivityTimes = imp5.getActivityTimes
    populateAllocatedTimeProperties = imp5.populateAllocatedTimeProperties
    timeAnalysisRollup = imp2.timeAnalysisRollup
    updateTimeManagementProperties = imp5.updateTimeManagementProperties


class ModelService(TcService):
    calculateCriticalPath = imp4.calculateCriticalPath
    computeAppearancePath = imp11.computeAppearancePath
    createFlow = imp4.createFlow
    editLogicalAssignments = imp4.editLogicalAssignments
    getCandidateToolsForToolRequirement = imp30.getCandidateToolsForToolRequirement
    getResolvedNodesFromLA = imp4.getResolvedNodesFromLA
    getToolRequirements = imp30.getToolRequirements
    removeFlow = imp4.removeFlow
    resolveLogicalAssignments = imp4.resolveLogicalAssignments
    resolveToolRequirement = imp30.resolveToolRequirement
    updateFlows = imp30.updateFlows
    validateScopeFlowsConsistency = imp47.validateScopeFlowsConsistency


class ResourceManagementService(TcService):
    checkToolParameters = imp6.checkToolParameters
    getStepP21FileCounts = imp37.getStepP21FileCounts
    getVendorCatalogInfo = imp37.getVendorCatalogInfo
    importStep3DModels = imp37.importStep3DModels
    importStepP21Files = imp37.importStepP21Files
    importVendorCatalogHierarchy = imp37.importVendorCatalogHierarchy
    updateNXToolAssemblies = imp37.updateNXToolAssemblies


class IPAManagementService(TcService):
    cleanIPATree = imp7.cleanIPATree
    deleteFilteredIPA = imp19.deleteFilteredIPA
    deletefilteredIPA = imp20.deletefilteredIPA
    doesIPAExist = imp7.doesIPAExist
    findAndRepopulateDynamicIPAs = imp24.findAndRepopulateDynamicIPAs
    generateIPATree = imp7.generateIPATree
    generateSearchScope = imp20.generateSearchScope
    getFilteredIPA = imp20.getFilteredIPA
    getFilteredIPAType = imp19.getFilteredIPAType
    localUpdateIPATree = imp7.localUpdateIPATree
    repopulateDynamicIPAs = imp24.repopulateDynamicIPAs
    saveSearchResult = imp20.saveSearchResult
    updateIPATree = imp7.updateIPATree


class StructureManagementService(TcService):
    closeAttachmentWindow = imp8.closeAttachmentWindow
    copyEBOPStructure = imp12.copyEBOPStructure
    copyRecursively = imp13.copyRecursively
    createCollabPlanningContext = imp15.createCollabPlanningContext
    createOrUpdateAttachments = imp8.createOrUpdateAttachments
    deleteAttachments = imp8.deleteAttachments
    findAffectedCCs = imp23.findAffectedCCs
    getAttachmentLineChildren = imp8.getAttachmentLineChildren
    getBOMLineActivities = imp8.getBOMLineActivities
    getBOMLineAttachments = imp8.getBOMLineAttachments
    getReferenceContexts = imp36.getReferenceContexts
    getStructureContextActivityLines = imp8.getStructureContextActivityLines
    getStructureContextLines = imp12.getStructureContextLines
    getStructureContextTopLines = imp8.getStructureContextTopLines
    moveAndResequenceNodes = imp43.moveAndResequenceNodes
    pasteDuplicateStructure = imp12.pasteDuplicateStructure
    pasteDuplicateStructure2 = imp13.pasteDuplicateStructure
    setReferenceContexts = imp36.setReferenceContexts


class MFGPropertyCollectorService(TcService):
    collectProperties = imp10.collectProperties


class StructureSearchService(TcService):
    createOrUpdateAssignmentRecipe = imp17.createOrUpdateAssignmentRecipe
    deleteAssignmentRecipes = imp17.deleteAssignmentRecipes
    findStudies = imp28.findStudies
    getAssignmentRecipes = imp17.getAssignmentRecipes
    nextSearch = imp44.nextSearch
    resolveAssignmentRecipe = imp17.resolveAssignmentRecipe
    searchConnectedLines = imp46.searchConnectedLines
    startSearch = imp44.startSearch
    stopSearch = imp44.stopSearch


class ValidationService(TcService):
    executeValidations = imp21.executeValidations
    getAllRegisteredCallbacks = imp29.getAllRegisteredCallbacks
    getAllValidations = imp21.getAllValidations
    getMaturityReport = imp33.getMaturityReport


class ImportExportService(TcService):
    exportToBriefcase = imp22.exportToBriefcase
    importFromBriefcase = imp39.importFromBriefcase
    importManufacturingFeatures = imp40.importManufacturingFeatures
    importManufacturingFeatures2 = imp41.importManufacturingFeatures
    importManufacturingFeatures3 = imp22.importManufacturingFeatures
    importManufaturingFeatures = imp42.importManufaturingFeatures


class CoreService(TcService):
    findCheckedOutsInStructure = imp25.findCheckedOutsInStructure
    findNodeInContext = imp26.findNodeInContext
    findNodeInContext2 = imp27.findNodeInContext
    getAffectedProperties = imp26.getAffectedProperties
    matchObjectsAgainstVariantRules = imp27.matchObjectsAgainstVariantRules


class ModelDefinitionsService(TcService):
    getManufacturingPropretyDescs = imp32.getManufacturingPropretyDescs
    getValidRelationTypes = imp32.getValidRelationTypes


class ConstraintsService(TcService):
    getPrecedenceConstraintPaths = imp35.getPrecedenceConstraintPaths
    getPrecedenceConstraints = imp35.getPrecedenceConstraints
    validateConstraintConsistency = imp35.validateConstraintConsistency
    validateProcessAreaAssignments = imp35.validateProcessAreaAssignments


class TimeWayPlanService(TcService):
    getTWPInformation = imp38.getTWPInformation
    setProductImage = imp38.setProductImage



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, PSBOMView, AssemblyArrangement, ItemRevision, BOMWindow, Item, ConfigurationContext
from typing import Dict, List
from tcsoa.gen.Cad._2007_01.StructureManagement import RevisionRuleConfigInfo
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateWindowsInfo3(TcBaseObj):
    """
    Main input structure that defines Item or ItemRevision of the top line in the BOMWindow. In the input, the
    BOMWindow is mutually exclusive with Item, ItemRevision and PSBOMView. If BOMWindow and Item, ItemRevision and
    PSBOMView objects are sent it will be considered as re-configure with BOMWindow. In the input, either
    revRuleConfigInfo object and objectsForConfigure object (variant rules or saved option set) or configContext object
    is required, the effGrpRevList object is used to re-configure BOMWindow if configContext object is not provided.
    
    :var clientId: Identifier that helps the client to track the objects created.
    :var bomWindow: BOMWindow object to be reconfigured. Must be NULLTAG when creating a new window.
    :var bomWinPropFlagMap: A map (string, string) of property names and respective value that needs to be set on
    window. User need to populate this map with following property string values as key and true or false as value,
    which will be set or unset in the window
    Valid property key values are:
    "show_unconfigured_variants"
    "show_unconfigured_changes"
    "show_suppressed_occurrences"
    "is_packed_by_default"
    "show_out_of_context_lines"
    "fnd0show_uncnf_occ_eff"
    "fnd0bw_in_cv_cfg_to_load_md".
    :var item: Item for top line of new window, or NULLTAG if itemRev is specified. Must be NULLTAG when reconfiguring
    an existing window.
    :var itemRev: ItemRevision for top line of new window, or NULLTAG if item is specified in which case the default
    revision will be used. Must be NULLTAG when reconfiguring an existing window.
    :var bomView: PSBOMView to be used when creating a new window, or NULLTAG to use the default view. Must be NULLTAG
    when reconfiguring an existing window.
    :var revRuleConfigInfo: Structure with information about RevisionRuleConfigInfo.
    :var objectsForConfigure: A list of VariantRule or StoredOptionSet stored option set object to set on this window.
    :var activeAssemblyArrangement: Active AssemblyArrangement of the BOMWindow.
    :var configContext: ConfigurationContext object reference.
    :var effGrpRevList: A list of Fnd0EffectivityGrp  objects, effGrpRevList is used along with BOMwindow and to
    configure or re-configure them.
    """
    clientId: str = ''
    bomWindow: BOMWindow = None
    bomWinPropFlagMap: StringMap2 = None
    item: Item = None
    itemRev: ItemRevision = None
    bomView: PSBOMView = None
    revRuleConfigInfo: RevisionRuleConfigInfo = None
    objectsForConfigure: List[BusinessObject] = ()
    activeAssemblyArrangement: AssemblyArrangement = None
    configContext: ConfigurationContext = None
    effGrpRevList: List[ItemRevision] = ()


"""
This is map of string Key to string Value.
"""
StringMap2 = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ListOfValues, ImanQuery
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SavedQueryClause(TcBaseObj):
    """
    Holds the definition of a single saved query clause.  A query clause represents a single attribute that the end
    user can supply as search criteria.
    
    :var attributeName: Name of the attribute.
    :var entryName: The L10N key used when displaying this attribute to the user.
    :var attributeType: Type of attribute as defined in the AttributeDescriptor definition.
    :var entryNameDisplay: The translated version of the L10N key that will be displayed to the user.
    :var logicalOperation: Logical operation used when evaluating the clause.  This can be "AND", "OR" or "".
    :var mathOperation: Math operation to use when evaluating the clause.  This can be "=", "!=", ">", ">=", "<", "<=",
    "IS_NULL" or "IS_NOT_NULL".
    :var defaultValue: Default value used for this attribute.
    :var lov: LOV used with this clause.
    :var attachedSpecifier: The attached specifier for the LOV.
    :var dependentIndexes: All the dependent indexes for this clause.
    :var keyLovId: Id of the key LOV for this attribute.
    """
    attributeName: str = ''
    entryName: str = ''
    attributeType: int = 0
    entryNameDisplay: str = ''
    logicalOperation: str = ''
    mathOperation: str = ''
    defaultValue: str = ''
    lov: ListOfValues = None
    attachedSpecifier: int = 0
    dependentIndexes: List[int] = ()
    keyLovId: int = 0


@dataclass
class SavedQueryDefinition(TcBaseObj):
    """
    Contains the clauses and sort attributes that make up a saved query definition.  Each of the search clauses and
    sort attributes in the definition are order dependent.  The order of the clause attributes indicates how the 'AND'
    and 'OR' logic of the saved query will be evaluated.  The order of the sort attributes indicates the precedence of
    the results sorting.
    
    :var query: Query definition on the server.
    :var clauses: The ordered set of clauses that make up the query definition.  Each clause represents an attribute
    where the user can enter a value.  Each of these values is used to make up the overall query.
    :var sortAttributes: Optional list of attributes to be used for default sorting.  The ascending or descending order
    is included in the definition.  The order that the attributes appear in the vector indicate the order that they
    will impact the sorting.
    """
    query: ImanQuery = None
    clauses: List[SavedQueryClause] = ()
    sortAttributes: List[SavedQuerySortAttribute] = ()


@dataclass
class SavedQuerySortAttribute(TcBaseObj):
    """
    Holds the definition of an attribute to be used for sorting the results of a saved query.  The order to be used in
    the sort is included along with the displayed name.
    
    :var attributeName: Name of the attribute.
    :var entryName: The L10N key used when displaying the attribute to the user.
    :var entryNameDisplay: The translated version of the L10N key.
    :var orderBy: The order to sort by.  This can either be "ASCENDING" or "DESCENDING".
    """
    attributeName: str = ''
    entryName: str = ''
    entryNameDisplay: str = ''
    orderBy: str = ''


@dataclass
class DescribeSavedQueryDefinitionInput(TcBaseObj):
    """
    Contains the input criteria required to find the saved query definition.  The main input here is the saved query
    object.  'true' should be the default substituteKeyword value.
    
    :var query: Saved query whos definition data is being requested.
    :var substituteKeyword: Indicates that keywords should be substituted when returning the definition.  Will return
    the attributes type display name instead of the raw defined default value for the clause. (For object_type and
    type_name attributes)
    """
    query: ImanQuery = None
    substituteKeyword: bool = False


@dataclass
class DescribeSavedQueryDefinitionsResponse(TcBaseObj):
    """
    Holds the detailed definition data for the requested saved queries.  Each saved query definition will include
    search clauses and default sort attributes.  Each search clause attribute will be shown to the user during a query,
    and will allow the user to enter a value.  Each clause can have a default value and operators indicating how the
    value is evaluated.  The sort attributes allow the saved query to define the default sorting to use when this query
    is executed.  This can be altered by the end user from the sort dialog.
    
    :var definitions: Saved query definitions data.  One definition is returned for each ImanQuery passed as input. 
    Each definition includes all the search criteria and sort attributes defined for each saved query.
    :var serviceData: Returned service data.
    """
    definitions: List[SavedQueryDefinition] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Query._2010_04.SavedQuery import FindSavedQueriesCriteriaInput, FindSavedQueriesResponse
from typing import List
from tcsoa.base import TcService


class SavedQueryService(TcService):

    @classmethod
    def findSavedQueries(cls, inputCriteria: List[FindSavedQueriesCriteriaInput]) -> FindSavedQueriesResponse:
        """
        The user can find the saved queries of interest by passing in the criteria such as query name and description.
        The queries that are matching the input criteria will be returned back to the user.  This operation can be sued
        to find the queries with a given name(s) or description(s) or combination of name(s) and description(s). This
        operation returns the queries matching the input criteria names and descriptions.
        
        Use cases:
        Find saved queries by given saved query name(s) and description(s).
        """
        return cls.execute_soa_method(
            method_name='findSavedQueries',
            library='Query',
            service_date='2010_04',
            service_name='SavedQuery',
            params={'inputCriteria': inputCriteria},
            response_cls=FindSavedQueriesResponse,
        )



from tcsoa.gen.RelationshipViewer._2012_10.services import NetworkEngineService as imp0
from tcsoa.gen.RelationshipViewer._2014_11.services import NetworkEngineService as imp1
from tcsoa.gen.RelationshipViewer._2019_12.services import NetworkEngineService as imp2
from tcsoa.base import TcService


class NetworkEngineService(TcService):
    createRelations = imp0.createRelations
    getGraphStyleDef = imp0.getGraphStyleDef
    getViews = imp0.getViews
    getViews2 = imp0.getViews2
    getViews3 = imp1.getViews3
    getViews4 = imp2.getViews4
    queryNetwork = imp0.queryNetwork
    queryNetwork2 = imp0.queryNetwork2



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class MultisiteDashboardResponse(TcBaseObj):
    """
    Return structure for getMultisiteDashBoardData operation.
    
    :var dashboardData: A map (string, string) of report id and dashboard data in JSON format  for the report recipe
    provided  by the user. The Dashboard data help to track and analyze the critical errors and key data points to
    monitor the health of Multi-Site operation. The dashboardData map uses id as key and data as value as listed below
    - recipeId      A unique identifier used for collating the input and output Multisite dashboard data
    - data    The dashboard data is stored in JSON format and will be used for displaying stack charts, bar chart, line
    charts and tables in dashboard interface.
    
    
    :var serviceData:  The standard ServiceData return.
    """
    dashboardData: StrStrMap = None
    serviceData: ServiceData = None


@dataclass
class ReportRecipe(TcBaseObj):
    """
    Structure containing detailed input information for the generation of Multi-Site dashbaord reports.
    
    :var recipeId: A unique identifier used for collating the input and output Multisite dashboard data.
    :var reportType: Name of different types of reports that need to be generated on server side. The supported report
    types are:
    
    - ItemErrorsBySite &ndash; Report the errors occurred during Multi-Site import, export and synchronization
    operations categorized by Teamcenter site name.
    - ItemErrorsByErrorType &ndash; Report the errors occurred during Multi-Site import, export and synchronization
    operations on each site categorized by error type.
    - ItemInconsistentOwnership- Report the items across all Teamcenter sites in Multi-Site federation which has
    varying ownership.
    
    
    :var filters: A map (string, list of strings) of report recipe filter names and values used for the generation of
    desired report type. the supported report recipe filter names and values are:
    
    - siteNames:-  List of names of Teamcenter sites on which the report needs to be generated on server side.
    - dateRange:-    List of dates in the date range for which the historic data is gathered. The dates are specified
    in MM-DD-YYYY format.
    
    """
    recipeId: str = ''
    reportType: str = ''
    filters: StrVecMap = None


"""
Map of string array property names to value.
"""
StrStrMap = Dict[str, str]


"""
Map of string array property names to values.
"""
StrVecMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, PSBOMView, AssemblyArrangement, VariantRule, ItemRevision, RevisionRule, Folder, User, CFMOverrideEntry, TC_Project, Item, ConfigurationContext
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from tcsoa.gen.Core._2012_09.ProjectLevelSecurity import TeamMemberInfo
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ModifyProjectsInfo2(TcBaseObj):
    """
    This structure holds the TC_Project object and the information required to modify the project.
    
    :var sourceProject: The TC_Project object to be modified.
    :var projectInfo: The information required to create the destination project.
    :var clientId: Unique identifier used by the client to track any errors.
    """
    sourceProject: TC_Project = None
    projectInfo: ProjectInformation2 = None
    clientId: str = ''


@dataclass
class OverrideInfo(TcBaseObj):
    """
    This contains information about the override RevisionRule entry
    
    :var ruleEntry: Refers to the CFMOverrideEntry of a RevisionRule object.
    :var folder: Refers to the Folder of override rule entry of RevisionRule object .
    """
    ruleEntry: CFMOverrideEntry = None
    folder: Folder = None


@dataclass
class PaginationInfo(TcBaseObj):
    """
    A Structure containing pagination criteria.
    
    :var startIndexForAvailableProjects: The start index to return the  available projects.
    :var maxToReturnForAvailableProjects: The maximum number of available available projects to return.
    """
    startIndexForAvailableProjects: int = 0
    maxToReturnForAvailableProjects: int = 0


@dataclass
class ProjectAssignOrRemoveInput(TcBaseObj):
    """
    
    Contains data used for assigning given set of objects to a list project objects for a given set of  configuration
    context.
    
    :var projectsToAssign: A list of TC_Project objects to for assignment.
    :var objectsForAssignment: A list of BusinessObject that needs to be assigned (added) to the given projects
    provided by projectsToAssing input parameter.
    :var projectsForRemoval: The TC_Project objects which needs to be removed from the given set of objects.
    :var objectsToRemoveFromProjects: A list of BusinessObject that needs to be removed from the given set of projects
    provided in projectsForRemoval parameter.
    :var contextInfo: A PropagationConfigurationContext structure.
    :var processAsynchronously: Flag indicating if this operation needs to be processed in asynchronously. A value True
    means to process it asynchronously. If the value is not set or is set to False then processing happens
    synchronously in the same request.
    """
    projectsToAssign: List[TC_Project] = ()
    objectsForAssignment: List[BusinessObject] = ()
    projectsForRemoval: List[TC_Project] = ()
    objectsToRemoveFromProjects: List[BusinessObject] = ()
    contextInfo: PropagationConfigurationContext = None
    processAsynchronously: bool = False


@dataclass
class ProjectInformation2(TcBaseObj):
    """
    Structure that holds the information required to create the destination project.
    
    :var projectId: The project ID of the project to be created.
    :var projectName: The name of the project to be created.
    :var projectDescription: The description of the project to be created.
    :var useProgramContext: The value of useProgramContext property in TC_Project.
    If value is true TC_Project is treated as program and if false it is project.
    :var active: The value of property  in TC_Project.
    If value is true than project is active so user can add/remove objects into this project. If value is false than
    project is no longer active. It will only available for viewing purpose. No further modification can be done on
    project.
    :var visible: The value of visible property in TC_Project.
    If value is true than this project will be visible to user in teamcenter otherwise project will be invisible.
    :var teamMembers: A list of TeamMemberInfo structures.
    :var clientId: Unique identifier used by the client to track any errors.
    :var propertyMap: A map of property names and desired values (string/string). This is map with property name and
    list of values in string format. The calling client is responsible for converting the different property types
    (int, float, date .etc) to a string using the appropriate functions in the SOA client framework's Property class.
    """
    projectId: str = ''
    projectName: str = ''
    projectDescription: str = ''
    useProgramContext: bool = False
    active: bool = False
    visible: bool = False
    teamMembers: List[TeamMemberInfo] = ()
    clientId: str = ''
    propertyMap: ProjectPropertiesMap = None


@dataclass
class ProjectOrLicenseDataInput(TcBaseObj):
    """
    Structure containing data for changing license and project data on WorkspaceObject.
    
    :var selectedObjects: A list of WorkspaceObject objects to be either attached or detached from selected list of
    Licenses or assigned or removed from selected list of projects
    :var selectedProjectsOrLicenses: A list of license IDs of ADA_License or Project IDs for TC_Project. These are
    strings of each with a maximum of 128 bytes size.
    :var eadParagraph: A list of authorizing paragraphs for the licenses being attached to WorkspaceObject objects.
    These are strings with a maximum of 80 bytes size. The size of eadParagraph vector should match the size of the
    selectedLicenses (each entry in eadParagraph maps to corresponding entry in selectedLicenses). If a paragraph entry
    is not applicable for a specific license (paragraph entries are applicable only for licenses of ITAR type), then
    that entry can be left blank. System will ignore any paragraph entry if it is not applicable for a license to be
    attached. This is an optional parameter. The eadParagraph is used for setting the ead_Paragraph attribute on
    WorkspaceObject.
    :var propertyName: The name of the property associated with the data it is either project_list or license_list.
    """
    selectedObjects: List[BusinessObject] = ()
    selectedProjectsOrLicenses: List[str] = ()
    eadParagraph: List[str] = ()
    propertyName: str = ''


@dataclass
class ProjectsInput(TcBaseObj):
    """
    A structure contain logged in user info, selected input objects list, configuration context info and pagination
    info.
    
    :var user: A user object, which is used to retrieve available projects to assign.
    :var selectedObjects: A list of Selected Input Objects.
    :var assignedObjects: A list of assigned objects from UI but not yet committed.
    :var paginationInfo: The pagination criteria.
    :var filterText: The filterText which should be applied for available projects to be returned.
    :var isAceContext: If true, the user is in ActiveWorkspace Content context; otherwise, false.
    """
    user: User = None
    selectedObjects: List[BusinessObject] = ()
    assignedObjects: List[TC_Project] = ()
    paginationInfo: PaginationInfo = None
    filterText: str = ''
    isAceContext: bool = False


@dataclass
class ProjectsOutput(TcBaseObj):
    """
    A list of projectsOutput objects.
    
    :var assignedProjectsList: A list of TC_Project objects which are assigned to the selected InputObject.
    :var selectedObjects: A list of selected input objects.
    """
    assignedProjectsList: List[TC_Project] = ()
    selectedObjects: List[BusinessObject] = ()


@dataclass
class ProjectsOutputResponse(TcBaseObj):
    """
    This structure holds the information for all assigned projects to the input objects and all the available projects.
    
    :var projectOutput: A list of projectsOutput objects.
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information. For this service, all objects are returned to the plain objects
    group. Error information will also be returned.
    :var availableProjectList: A list of available TC_Project objects sorted in  alphabetical order.
    :var projectOptions: A map(string, list of ConfigurationContextChoice) of ActiveWorkspace Content context
    attributes with the list of ActiveWorkspace Content context attributes possible values.
    :var totalFound: The total number of projects found.
    :var totalLoaded: The total number of projects loaded.
    :var endIndex: The end index to return the projects.
    """
    projectOutput: ProjectsOutput = None
    serviceData: ServiceData = None
    availableProjectList: List[TC_Project] = ()
    projectOptions: ProjectOptions = None
    totalFound: int = 0
    totalLoaded: int = 0
    endIndex: int = 0


@dataclass
class PropagationConfigurationContext(TcBaseObj):
    """
    PropagationConfigurationContext contains data which can be applied to assign to projects and remove from projects
    operation.
    
    :var selectedTopLevelObject: Selected top level object in context of ACE ( Active Content Expericence ), if this
    object is passed we will fetch variant rule and revision rule.
    :var variantRule: 
    The VariantRule in context of the assign to project  or attach to license operation.
    :var revisionRule: The RevisionRule associated with the assign to project or attach to license operation.
    :var typeOption: An integer indicating the type of Item or Item Revision, valid values are 0 for Item and 1 for
    Item Revision.
    :var depth: A number indicating how deep the traversal needs to be performed for a given structure, applicable only
    for structures.
    """
    selectedTopLevelObject: BusinessObject = None
    variantRule: VariantRule = None
    revisionRule: RevisionRule = None
    typeOption: int = 0
    depth: int = 0


@dataclass
class PropagationDataInput(TcBaseObj):
    """
    Structure selected objects, configuration information which will be used for propagation and a structure of
    ReconstructBOMWindowInfo.
    
    :var assignOrAttachData: A list of projects to be assigned to a given set of objects or list of licenses that needs
    to be attached to a given set of objects.
    :var removeOrDetachData: A list containing license to be detached from a given set of objects or projects to be
    removed from a given set of objects.
    :var propertyDataToSetAndPropagate: A list of selected business object with property value to be removed or added.
    :var configInfo: The configuration information for propagation.
    :var reconstructConfigurationInfo: Information that will be used to reconstruct the given BOMWindow if the
    operation needs to be performed in a different process.
    """
    assignOrAttachData: List[ProjectOrLicenseDataInput] = ()
    removeOrDetachData: List[ProjectOrLicenseDataInput] = ()
    propertyDataToSetAndPropagate: List[PropertyChangeData] = ()
    configInfo: PropagationConfigurationContext = None
    reconstructConfigurationInfo: ReconstructBOMWindowsInfo = None


@dataclass
class PropertyChangeData(TcBaseObj):
    """
    A Structure containing the name of property which needs to be modified, the new intended value after modification
    and set of selected objects.
    
    :var selectedObjects: A list of selected objects for which the property needs to be changed.
    :var propertyName: The name of propagation enabled property for modification.
    :var newPropertyValues: A list of values that are need to be set for the given property. This is the intended
    property values after the change.
    """
    selectedObjects: List[BusinessObject] = ()
    propertyName: str = ''
    newPropertyValues: List[str] = ()


@dataclass
class ReconstructBOMWindowsInfo(TcBaseObj):
    """
    This contains the list of BOMWindow objects and the corresponding RevisionRule and VariantRule objects or
    StoredOptionSet object information.
    
    :var item: Item object reference.
    :var itemRev: ItemRevision object reference.
    :var bomView: PSBOMView object reference.
    :var objectsForConfigure: List of variant rules or single stored option set object to set on this window.
    :var revRuleConfigInfo: Refers to a RevisionRuleConfigInfo struct object.
    :var activeAssemblyArrangement: Active assembly arrangement of this BOMWindow.
    :var configContext: ConfigurationContext object reference.
    :var bomWinPropFlagMap: Mapping for window property and respective value that needs to be set on window. User need
    to populate this map with following property string values as key and true or false as value, which will be set or
    unset on the window. Valid property values are
    """
    item: Item = None
    itemRev: ItemRevision = None
    bomView: PSBOMView = None
    objectsForConfigure: List[BusinessObject] = ()
    revRuleConfigInfo: RevisionRuleConfigInfo = None
    activeAssemblyArrangement: AssemblyArrangement = None
    configContext: ConfigurationContext = None
    bomWinPropFlagMap: StringMap = None


@dataclass
class RevisionRuleConfigInfo(TcBaseObj):
    """
    This contains the RevisionRule object and a RevisionRuleEntryProps  structure.
    
    :var revRule: The RevisionRule object used for configuration of this BOMWindow object.
    :var props: Refers to RevisionRuleEntryProps struct object.
    """
    revRule: RevisionRule = None
    props: RevisionRuleEntryProps = None


@dataclass
class RevisionRuleEntryProps(TcBaseObj):
    """
    This contains information about the RevisionRule entry properties.
    
    :var unitNo: The unit number of RevisionRule object.
    :var date: The date of RevisionRule object.
    :var today:  Refers to a flag to indicate that the date is today on RevisionRule object.
    :var endItem: The  Item that indicates end item for RevisionRule object.
    :var endItemRevision: The ItemRevision that indicates end item revision for RevisionRule object.
    :var overrideFolders: A list of OverrideInfo struct.
    """
    unitNo: int = 0
    date: datetime = None
    today: bool = False
    endItem: Item = None
    endItemRevision: ItemRevision = None
    overrideFolders: List[OverrideInfo] = ()


@dataclass
class ConfigurationContextChoice(TcBaseObj):
    """
    This structure contains context related info.
    
    :var internalValue: Internal value of business object.
    :var displayValue: Display value of business object.
    :var isDefaultValue: If true, this choice is set to default; otherwise, false.
    """
    internalValue: str = ''
    displayValue: str = ''
    isDefaultValue: bool = False


@dataclass
class CopyProjectsInfo2(TcBaseObj):
    """
    Structure that holds project information required to create a new TC_Project object using this operation.
    
    :var sourceProject: The TC_Project object needs to be copied.
    :var projectInfo: The information required to copy the project.
    :var clientId: Unique identifier used by the client to track any errors.
    """
    sourceProject: TC_Project = None
    projectInfo: ProjectInformation2 = None
    clientId: str = ''


"""
A map(string/list of ConfigurationContextChoice) of ActiveWorkspace Content context attributes with the list of ActiveWorkspace Content context attributes possible values.
"""
ProjectOptions = Dict[str, List[ConfigurationContextChoice]]


"""
A map (string, string) of property names and desired values. The calling client is responsible for converting the different property types (int, float, date .etc) to a string using the appropriate functions in the SOA client framework's Property class.
"""
ProjectPropertiesMap = Dict[str, List[str]]


"""
 A StringMap which holds string as key and value as string.
"""
StringMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TCCalendar, ScheduleTask, Schedule, ResourceAssignment, TaskDependency, TCCalendarEvent
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class KeyValueContainer(TcBaseObj):
    """
    The key value container.
    
    :var key: An integer key identifying the attribute to be updated or parameterised index
    :var value: A string representation of the new value of the attribute.
    :var type: An integer to help determine the type of object represented by the value.
    """
    key: int = 0
    value: str = ''
    type: int = 0


@dataclass
class MultipleScheduleLoadResponse(TcBaseObj):
    """
    A container for the load schedule response.
    
    :var serviceData: The ServiceData.
    :var scheduleData: A collection of the schedule load responses.
    """
    serviceData: ServiceData = None
    scheduleData: List[ScheduleLoadResponse] = ()


@dataclass
class ScheduleChangeContainer(TcBaseObj):
    """
    A container for updating a schedule.
    
    :var scheduleUpdates: The updates for the schedule.
    :var newTasks: The information needed to create new tasks.
    :var newDependencies: The information needed to create new dependencies.
    :var newAssignments: The information needed to create new resource assignments.
    :var taskUpdates: The information needed to update existing tasks.
    :var dependencyUpdates: The information needed to update existing dependencies.
    :var assignmentUpdates: The information needed to update existing resource assignments.
    :var deletedTasks: The list of tasks to delete.
    :var deletedDependencies: The list of dependencies to delete.
    :var deletedAssignments: The list of resource assignments to delete.
    """
    scheduleUpdates: UpdateContainer = None
    newTasks: List[CreateTaskContainer] = ()
    newDependencies: List[CreateTaskDependencyContainer] = ()
    newAssignments: List[CreateResourceAssignmentContainer] = ()
    taskUpdates: List[UpdateContainer] = ()
    dependencyUpdates: List[UpdateContainer] = ()
    assignmentUpdates: List[UpdateContainer] = ()
    deletedTasks: List[ScheduleTask] = ()
    deletedDependencies: List[TaskDependency] = ()
    deletedAssignments: List[ResourceAssignment] = ()


@dataclass
class ScheduleLoadResponse(TcBaseObj):
    """
    A container for the load schedule response.
    
    :var schedule: The schedule.
    :var scheduleTask: All the tasks in this schedule.
    :var resourceAssignment: All the resource assignments in this schedule.
    :var taskDependency: All the task dependencies in this schedule.
    :var calendar: All the calendars referenced in this schedule.
    :var calendarEvent: All the calendar events referenced in the calendars.
    """
    schedule: Schedule = None
    scheduleTask: List[ScheduleTask] = ()
    resourceAssignment: List[ResourceAssignment] = ()
    taskDependency: List[TaskDependency] = ()
    calendar: List[TCCalendar] = ()
    calendarEvent: List[TCCalendarEvent] = ()


@dataclass
class StringValueContainer(TcBaseObj):
    """
    A container which represents a single attribute's value with a String for a key.
    
    :var key: A string key identifying the attribute.
    :var value: A string representation of the value of the attribute.
    :var type: An integer to help determine the type of object represented by the value.
    """
    key: str = ''
    value: str = ''
    type: int = 0


@dataclass
class UpdateContainer(TcBaseObj):
    """
    A container holder all the updates for a single object.
    
    :var object: The object being updated.
    :var keyValueContainer: The collection of KeyValueContainers.
    :var stringValueContainer: The collection of StringValueContainers.
    """
    object: BusinessObject = None
    keyValueContainer: List[KeyValueContainer] = ()
    stringValueContainer: List[StringValueContainer] = ()


@dataclass
class CreateResourceAssignmentContainer(TcBaseObj):
    """
    A container to create a new ResourceAssignment.
    
    :var taskID: The UID of the task.
    :var resource: The UID of the User or Discipline.
    :var percentage: The percentage assigned.
    :var discipline: Resource's discipline ID (can be blank).
    :var stringValueContainer: A collection of additional attributes.
    """
    taskID: str = ''
    resource: str = ''
    percentage: float = 0.0
    discipline: str = ''
    stringValueContainer: List[StringValueContainer] = ()


@dataclass
class CreateTaskContainer(TcBaseObj):
    """
    A container for the creation of a new task.
    
    :var id: A unique ID for the task.
    :var name: The name.
    :var startDate: The start date.
    :var finishDate: The finsh date.
    :var startDateOffset: NOT USED
    :var finishDateOffset: NOT USED
    :var workEstimate: The number of minutes of estimated work.
    :var workComplete: The number of minutes of completed work.
    :var percentComplete: The % complete of the task.
    :var duration: The minute duration of the task.
    :var stringValueContainer: A collection of additional attributes.
    :var description: The description.
    :var parentTaskID: The UID of the task's parent.
    :var prevSiblingID: The UID of the task directly before this task.
    :var priority: The priority.
    :var fixedType: The fixed type.
    :var autoComplete: Is autocomplete?
    :var taskType: The task type.
    :var constraint: The constraint.
    """
    id: str = ''
    name: str = ''
    startDate: datetime = None
    finishDate: datetime = None
    startDateOffset: int = 0
    finishDateOffset: int = 0
    workEstimate: int = 0
    workComplete: int = 0
    percentComplete: float = 0.0
    duration: int = 0
    stringValueContainer: List[StringValueContainer] = ()
    description: str = ''
    parentTaskID: str = ''
    prevSiblingID: str = ''
    priority: int = 0
    fixedType: int = 0
    autoComplete: bool = False
    taskType: int = 0
    constraint: int = 0


@dataclass
class CreateTaskDependencyContainer(TcBaseObj):
    """
    A container for the creation of a new task dependency.
    
    :var predecessorTaskID: The UID of the predecessor Task.
    :var successorTaskID: The UID of the sucessor Task.
    :var type: The type of dependency.
    :var lag: The number of minutes lag time.
    :var stringValueContainer: A collection of additional attributes.
    """
    predecessorTaskID: str = ''
    successorTaskID: str = ''
    type: int = 0
    lag: int = 0
    stringValueContainer: List[StringValueContainer] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Participant._2018_11.Participant import ParticipantInfoInput, ParticipantInfo, ReassignParticipantResponse, ReassignParticipantInfoInput, ParticipantResponse
from tcsoa.base import TcService


class ParticipantService(TcService):

    @classmethod
    def reassignParticipants(cls, input: List[ReassignParticipantInfoInput]) -> ReassignParticipantResponse:
        """
        Reassigns the participant roles from one user to another for a given list of participant types on the input
        list of objects. The Particpant for the fromAssignee user is replaced with the Participant for the toAssignee
        user. If the toAssignee user already exists as participant, then the fromAssignee user will not be replaced.
        
        Use cases:
        In Active Workspace Client, it will be required to reassign participants of a particular Participant Types from
        an old assignee value to new assignee on WorkspaceObject.
        """
        return cls.execute_soa_method(
            method_name='reassignParticipants',
            library='Participant',
            service_date='2018_11',
            service_name='Participant',
            params={'input': input},
            response_cls=ReassignParticipantResponse,
        )

    @classmethod
    def removeParticipants(cls, input: List[ParticipantInfo]) -> ParticipantResponse:
        """
        Removes participants from the WorkspaceObject. The participants can be removed from EPMJob or other
        WorkspaceObject objects that are provided as input
        
        Use cases:
        In the Active Workspace Client, participants added to workflow can be removed from the Participant page of the
        EPMTask in that workflow. The participants added to any WorkspaceObject can be removed from the participant
        page of the WorkspaceObject.
        """
        return cls.execute_soa_method(
            method_name='removeParticipants',
            library='Participant',
            service_date='2018_11',
            service_name='Participant',
            params={'input': input},
            response_cls=ParticipantResponse,
        )

    @classmethod
    def addParticipants(cls, input: List[ParticipantInfoInput]) -> ParticipantResponse:
        """
        Creates Participant objects and adds them to the input WorkspaceObject. These participants can be added to
        WorkspaceObject objects that are targets in a workflow or the workflow (EPMJob) itself. The participants can
        then be used by workflow handlers to assign tasks during workflow execution.
        
        Use cases:
        In Active Workspace Client, this operation can be used for following two cases:
        
        1.    Adding participants to WorkspaceObject from Participants tab.
        2.    Adding participants to a Workflow (EPMJob) from the Participants tab shown when a task is selected. 
        
        The Submit to workflow panel can be used to assign participants configured in the workflow.  The participants
        selected in the panel can be added based on the configuration to either the EPMJob or the first target in the
        workflow using this operation.
        """
        return cls.execute_soa_method(
            method_name='addParticipants',
            library='Participant',
            service_date='2018_11',
            service_name='Participant',
            params={'input': input},
            response_cls=ParticipantResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, User
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class EmailInfo(TcBaseObj):
    """
    This structure stores email related information.
    
    :var subject: The subject of email message.
    :var messageBody: The body of the email message.
    :var listOfRecipients: List of internal Teamcenter recipients. These recipients can be users, groups and address
    lists. Email will be sent to all the users representing the recipients based on the email address provided. If
    there is no email address specified for any user, email will be sent to the OS user name for the user.
    :var listOfCcRecipients: List of internal Teamcenter CC recipients. These recipients can be users, groups and
    address lists. Email will be sent to all the users representing the recipients based on the email address provided.
    If there is no email address specified for any user, email will be sent to the OS user name for the user.
    :var listOfExtRecipients: List of external recipients' email addresses.
    :var listOfExtCcRecipients: CC recipients' email addresses.
    :var fmsReadTickets: FMS read tickets for the files to be attached to the email. This is an optional parameter.
    :var sender: The sender of the email message. The sender is a Teamcenter user.
    :var clientID: ID to uniquely identify the email transaction.
    """
    subject: str = ''
    messageBody: str = ''
    listOfRecipients: List[BusinessObject] = ()
    listOfCcRecipients: List[BusinessObject] = ()
    listOfExtRecipients: List[str] = ()
    listOfExtCcRecipients: List[str] = ()
    fmsReadTickets: List[str] = ()
    sender: User = None
    clientID: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, RevisionRule
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class PasteDuplicateInput(TcBaseObj):
    """
    A structure containing source BOMLine and target BOMLine as input for pasteDuplicateStructure operation.
    
    :var sourceLine: Source BOMLine to be cloned.
    :var targetLine: Target BOMLine under which the cloned source BOMLine is to be pasted
    """
    sourceLine: BusinessObject = None
    targetLine: BusinessObject = None


@dataclass
class PasteDuplicateStructureOutput(TcBaseObj):
    """
    A structure containing the target BOMLine, the new child BOMLine created under this target and the index of
    corresponding source BOMLine in the input.
    
    :var srcLineIndex: Index of the source BOMLine in the input data.
    :var targetLine: Target BOMLine under which the new cloned BOMLine is pasted.
    :var newChildLine: New BOMLine which is cloned from the source BOMLine and is pasted under the target BOMLine.
    """
    srcLineIndex: int = 0
    targetLine: BusinessObject = None
    newChildLine: BusinessObject = None


@dataclass
class PasteDuplicateStructureResponse(TcBaseObj):
    """
    Response for pasteDuplicateStructure operation.
    
    :var pasteDuplicateOutput: The target BOMLine, the new child BOMLine created under this target and the index of
    corresponding source BOMLine in the input.
    :var serviceData: Partial errors as part of the serviceData.
    """
    pasteDuplicateOutput: List[PasteDuplicateStructureOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CopyRecursivelyConfigurationInfo(TcBaseObj):
    """
    Structure containing configuration information.
    
    :var revisionRule: The RevisionRule with which the template BOMWindow is configured for cloning.
    :var variantRules: A list of variant rules to configure the BOMWindow for cloning. List can contain either
    StoredOptionSet or VariantRule. Both type of variant rules are not supported in the same list. If the list is empty
    all the SVRs created on the BOMWindow are carried over to the new BOMWindow.
    :var copySuppressedLines: If true, the  suppressed lines are carried to the cloned structure otherwise not.
    :var copyFutureEffectivities: If true, the furture effectivites are carried to the cloned structure otherwise not.
    :var additionalInfo: A generic structure to be used to capture additional information.
    """
    revisionRule: RevisionRule = None
    variantRules: List[BusinessObject] = ()
    copySuppressedLines: bool = False
    copyFutureEffectivities: bool = False
    additionalInfo: AdditionalInfo2 = None


@dataclass
class CopyRecursivelyInputInfo(TcBaseObj):
    """
    Holds information about the original structure to be cloned, BOMWindow configuration used to clone the structure
    and the information for the cloned structure.
    
    :var templateInfo: Structure with the information about the template and rule to be used for cloning.
    :var configurationInfo: Structure containing configuration information.
    :var newObjectInfo: Structure containing information about the cloned object.
    """
    templateInfo: CopyRecursivelyTemplateInfo = None
    configurationInfo: CopyRecursivelyConfigurationInfo = None
    newObjectInfo: CopyRecursivelyNewObjectInfo = None


@dataclass
class CopyRecursivelyNewObjectInfo(TcBaseObj):
    """
    Structure containing information about the cloned object.
    
    :var newName: The name of cloned BOMLine object.
    :var newDescription: The description of cloned object.
    :var newId: Id of cloned BOMLine object.
    :var newrevId: The revision id of cloned object.
    :var pasteTarget: The target BOMLine object under which the cloned object is created. If NULL, the cloned object is
    created as new root in a new BOMWindow object.
    :var additionalInfo: Any additional information.Reserved for future use.
    """
    newName: str = ''
    newDescription: str = ''
    newId: str = ''
    newrevId: str = ''
    pasteTarget: BusinessObject = None
    additionalInfo: AdditionalInfo2 = None


@dataclass
class CopyRecursivelyResponse(TcBaseObj):
    """
    The response is a structure containing the map of template object and the item revision of cloned object, a fms
    file ticket that captures logs.
    
    :var dataMap: A Map <BusinessObject, BusinessObject> contains the input object which has been cloned as key and the
    item revision of the cloned object as value.
    :var logFileTicket: The FMS ticket for the transient file that captures the log of cloning operation.
    :var serviceData: The standard service data.
    """
    dataMap: OriginalToCloneStructureMap = None
    logFileTicket: str = ''
    serviceData: ServiceData = None


@dataclass
class CopyRecursivelyTemplateInfo(TcBaseObj):
    """
    Structure with the information about the template and rule to be used for cloning.
    
    :var objectToClone: The object to be cloned. The valid object types are: Mfg0BvrPart, Mfg0BvrWorkarea,
    Mfg0BvrProcess and Mfg0BvrOperation.
    :var cloningRule: The preference name holding the list of caluses to be  used for cloning.
    :var referenceWindow: BOMWindow  from where the objects are referred during cloning. If NULL, the objects are
    referred from the template BOMWindow.
    :var additionalInfo: Structure for additional information. Reserved for future use.
    """
    objectToClone: BusinessObject = None
    cloningRule: str = ''
    referenceWindow: List[BusinessObject] = ()
    additionalInfo: AdditionalInfo2 = None


@dataclass
class AdditionalInfo2(TcBaseObj):
    """
    A structure containg the maps to capture the additional information.
    
    :var strToDateVectorMap: A map of string to list of dates.
    :var strToDoubleVectorMap: A map of string to list if doubles.
    :var strToStrVectorMap: A map of string to list if strings.
    :var strToObjVectorMap: A map of string to list of business object.
    :var strToIntVectorMap: A map of string to list of integers.
    """
    strToDateVectorMap: StringToDateVectorMap2 = None
    strToDoubleVectorMap: StringToDoubleVectorMap2 = None
    strToStrVectorMap: StringToStringVectorMap2 = None
    strToObjVectorMap: StringToObjectVectorMap2 = None
    strToIntVectorMap: StringToIntegerVectorMap2 = None


"""
A Map <BusinessObject, BusinessObject> contins the input object which has been cloned as key and the item revision of the cloned object as value.
"""
OriginalToCloneStructureMap = Dict[BusinessObject, BusinessObject]


"""
A map of string to list of dates.
"""
StringToDateVectorMap2 = Dict[str, List[datetime]]


"""
A map of string to list of doubles.
"""
StringToDoubleVectorMap2 = Dict[str, List[float]]


"""
A map of string to list of integers.
"""
StringToIntegerVectorMap2 = Dict[str, List[int]]


"""
A map of string to list of business objects.
"""
StringToObjectVectorMap2 = Dict[str, List[BusinessObject]]


"""
A map of string to list of strings.
"""
StringToStringVectorMap2 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindObjectsCriteria(TcBaseObj):
    """
    Criteria to find objects
    
    :var className: Class name to search by.
    :var searchAttributes: Attributes and values to search by.
    """
    className: str = ''
    searchAttributes: FindObjectsAttrAndValueMap = None


@dataclass
class FindObjectsInput2(TcBaseObj):
    """
    Input structure for FindObjectsByClassAndAttributes2
    
    :var clientId: Client unque id.
    :var startIndex: The start index
    :var maxLoad: Maximum objects to load.
    :var maxToReturn: Maximum uids to return.
    :var searchCriteria: Search criteria
    :var searchMode: Type of search.
    :var uids: A list of object uids to inflate.
    :var attributes: A set of attributes to inflate.
    """
    clientId: str = ''
    startIndex: int = 0
    maxLoad: int = 0
    maxToReturn: int = 0
    searchCriteria: List[FindObjectsCriteria] = ()
    searchMode: SearchMode = None
    uids: List[BusinessObject] = ()
    attributes: List[str] = ()


@dataclass
class FindObjectsResponse2(TcBaseObj):
    """
    Returns a structure of data.
    
    :var result: The list of business objects
    :var totalLoaded: Total number of objects loaded.
    :var totalFound: Total number of objects found.
    :var thumbnailMap: A map containing the thumbnail file tickets.
    :var serviceData: The service data.
    """
    result: List[BusinessObject] = ()
    totalLoaded: int = 0
    totalFound: int = 0
    thumbnailMap: ThumbnailMap = None
    serviceData: ServiceData = None


@dataclass
class FindUsersTasksResponse(TcBaseObj):
    """
    A structure containing the user's tasks.
    
    :var tasksToPerform: A list of tasks to perform.
    :var tasksToTrack: A list of tasks to track.
    :var serviceData: Service data.
    """
    tasksToPerform: List[BusinessObject] = ()
    tasksToTrack: List[BusinessObject] = ()
    serviceData: ServiceData = None


class SearchMode(Enum):
    """
    Type of search to perform.
    """
    GeneralInListQuery = 'GeneralInListQuery'
    GeneralQuery = 'GeneralQuery'
    SavedQuery = 'SavedQuery'


"""
Attribute and value map
"""
FindObjectsAttrAndValueMap = Dict[str, str]


"""
Used to keep track of a group of business objects and thumbnail file tickets.
"""
ThumbnailMap = Dict[BusinessObject, str]



from __future__ import annotations

from tcsoa.gen.ConfigFilterCriteria._2011_06.EffectivityManagement import ConfigFormula, ConfigurableProductsResponse, ConfigExpression, EffectivityTable, EffectivityConditionResponse, EffectivityTablesResponse, RevRuleEffectivityCriteriaResponse, EffectivityFormulaeResponse, AvailableProductEffectivityResponse, EffectivityExpressionsResponse
from tcsoa.gen.BusinessObjects import ConfigurationFamily, POM_object, RevisionRule
from typing import List
from tcsoa.base import TcService


class EffectivityManagementService(TcService):

    @classmethod
    def getRevRuleEffectivityCriteria(cls, revisionRules: List[RevisionRule], substituteDependentVariables: int, applyConstraints: int, applyDefaults: int, productName: str, productNameSpace: str, configuratorURL: str) -> RevRuleEffectivityCriteriaResponse:
        """
        This operation returns the effectivity criteria associated with a set of RevisionRule objects. If a non zero
        value is specified for parameters 'substituteDependentVariables', 'applyConstraints', and/or 'applyDefaults',
        the operation will connect to the effectivity configurator specified by parameter 'configuratorURL'. The actual
        evaluation of configurator rules, and conversions to and from formula strings, are performed by this
        configurator service. The resulting formula string conversions may vary depending on the choice of service
        endpoint. For example a given configurator might return shorthand representations for formulae if these are
        unique in the context of the specified product. Teamcenter 9 only supports local builtin Teamcenter
        configurators where parameter 'configuratorURL' can be set to an empty string. Teamcenter configurators used in
        the EffectivityManagement service interface encode configuration formulae in Explicit Teamcenter Language for
        which no product context is required (see operation 'getEffectivityExpressions' for more details on Explicit
        Teamcenter Language and 'productName' and 'productNameSpace'.
        
        Operation 'getRevRuleEffectivityCriteria' should be used if symbolic variable substitution and/or constraint
        processing is required for *all* variant option families, or if validation records are required that specify
        violated constraints.
        
        
        Use cases:
        Use case 1:
        Initialize an effectivity criteria dialog for a given RevisionRule.
        - The application uses operation 'getRevRuleEffectivityCriteria' with 'revisionRules={myRevRule}, productName=,
        productNameSpace=', 'substituteDependentVariables=0, applyConstraints=0.'
        - The response will contain effectivity ranges as they are stored on RevisionRule 'myRevRule'  without
        additional configurator processing. References to effectivity configuration families (ConfigurationFamily) are
        returned in 'ServiceData'.
        
        
        
        Use case 2:
        Initialize an effectivity criteria dialog for a product that was created using the Teamcenter effectivity
        configurator with the intent to setup a new RevisionRule.
        - The application uses operation 'getRevRuleEffectivityCriteria' with 'productName=MyProductItemID,
        productNameSpace=MyProductRevID, substituteDependentVariables=0, applyConstraints=0.'
        - The response will contain all nominal effectivity ranges in this product. No constraint processing is
        required. References to effectivity configuration families (ConfigurationFamily) are returned in 'ServiceData'.
        
        
        
        Use case 3:
        Initialize an effectivity criteria dialog for a product that is associated with a Collaborative Design (CD)
        model with the intent to setup a new RevisionRule.
        - Identifiers for the product associated with the CD (Cpd0CollaborativeDesign) are obtained from CD  properties
        'mdl0config_product_name', and 'mdl0config_prod_namespace'.
        - The application uses operation 'getRevRuleEffectivityCriteria' with 'productName= , productNameSpace=  as
        obtained in the previous step, substituteDependentVariables=0, applyConstraints=0.'
        - The response will contain all nominal effectivity ranges in this product. No constraint processing is
        required. References to effectivity configuration families (ConfigurationFamily) are returned in 'ServiceData'.
        
        """
        return cls.execute_soa_method(
            method_name='getRevRuleEffectivityCriteria',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'revisionRules': revisionRules, 'substituteDependentVariables': substituteDependentVariables, 'applyConstraints': applyConstraints, 'applyDefaults': applyDefaults, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=RevRuleEffectivityCriteriaResponse,
        )

    @classmethod
    def convertEffectivityExpressions(cls, expressions: List[ConfigExpression], productName: str, productNameSpace: str, configuratorURL: str) -> EffectivityFormulaeResponse:
        """
        This operation returns string representations (formulae) for a given set of effectivity expressions. This
        operation connects to the effectivity configurator service whose service endpoint is specified by the
        'configuratorURL' parameter. The actual conversion to formula strings is performed by this configurator
        service. The result may vary depending on the choice of service endpoint and the product identified by
        parameters 'productName' and 'productNameSpace'. For example a given configurator might return shorthand
        representations for formulae if these are unique in the context of the specified product. Teamcenter 9 only
        supports local built in Teamcenter configurators where parameters 'productName', 'productNameSpace', and'
        configuratorURL' can be set to an empty string.
        
        
        Use cases:
        Obtain a formula string representation ('ConfigFormula') in the context of a Collaborative Design model
        (Cpd0CollaborativeDesign) for an effectivity expression ('ConfigExpression') when these expression do not
        already reference a formula.
        
        When effectivity expressions are returned from effectivity SOA operations, e.g.
        'getAvailableProductEffectivity' for a given RevisionRule, they usually already contain a corresponding formula
        string representation. However, there is no guarantee they always do, e.g. if the configurator link is
        temporarily down.
        
        Another scenario in which effectivity expressions exist without a corresponding formula string representation
        is when these expressions are constructed in the SOA client.
        - Identifiers for the product associated with the Collaborative Design model (CD) are obtained from CD
        properties  'mdl0config_product_name', and 'mdl0config_prod_namespace'.
        - Operation 'convertEffectivityExpressions' is used to convert the effectivity conditions into formula strings.
        
        """
        return cls.execute_soa_method(
            method_name='convertEffectivityExpressions',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'expressions': expressions, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=EffectivityFormulaeResponse,
        )

    @classmethod
    def convertEffectivityTables(cls, effectivityTables: List[EffectivityTable], productName: str, productNameSpace: str, configuratorURL: str) -> EffectivityFormulaeResponse:
        """
        This operation returns string representations (formulae) for a given set of effectivity tables where each table
        describes a separate set of effectivity ranges. This operation connects to the effectivity configurator service
        whose service endpoint is specified by the 'configuratorURL' parameter. The actual conversion to formula
        strings is performed by this configurator service. The result may vary depending on the choice of service
        endpoint and the product identified by parameters 'productName' and 'productNameSpace'. For example a given
        configurator might return shorthand representations for formulae if these are unique in the context of the
        specified product. Teamcenter 9 only supports local built in Teamcenter configurators where parameters
        'productName', 'productNameSpace', and 'configuratorURL' can be set to an empty string.
        
        Use cases:
        Review the configurator encoded formula string representation for a table of effectivity ranges before
        assigning the effectivity ranges to a Revision Rule.
        - An application displays a dialog that collects a set of effectivity ranges in form of(unit in ,unit out )
        and/or (date in ,date out) tuples from the user with the intent to eventually use these effectivity ranges as
        configuration criteria to be assigned to a RevisionRule.
        - The user wants to review the configurator encoding of the corresponding effectivity expression
        - Operation 'convertEffectivityTables' is used to convert the effectivity table of effect in and effect out
        points into a formula string.
        
        """
        return cls.execute_soa_method(
            method_name='convertEffectivityTables',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'effectivityTables': effectivityTables, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=EffectivityFormulaeResponse,
        )

    @classmethod
    def setEffectivityConditions(cls, sampleObjects: List[POM_object], formulae: List[ConfigFormula], expressions: List[ConfigExpression], opCode: int, actionCode: int, affectedObjects: List[POM_object], replacedObjects: List[POM_object], productName: str, productNameSpace: str, configuratorURL: str) -> EffectivityConditionResponse:
        """
        This operation applies the specified effectivity condition to all objects listed in 'affectedObjects' and their
        sub objects (where applicable) in the context of the given ('productName,productNameSpace)' combination. 
        The operation returns formulae for created/modified effectivity conditions in a configurator specific encoding
        along with a reference to the modified objects. This operation connects to the effectivity configurator service
        whose service endpoint is specified by the 'configuratorURL' parameter. The actual conversion to and from
        formula strings is performed by this configurator service. The result may vary depending on the choice of
        service endpoint. For example a given configurator might return shorthand representations for formulae if these
        are unique in the context of the specified product. Teamcenter 9 only supports local built in Teamcenter
        configurators where parameter 'configuratorURL' can be set to an empty string. Teamcenter configurators used in
        the 'EffectivityManagement' service interface encode configuration formulae in Explicit Teamcenter Language for
        which no product context is required (see operation 'getEffectivityExpressions' for more details on Explicit
        Teamcenter Language  and 'productName and productNameSpace').
        
        If the effectivity condition are propagated to sub elements (e.g. sub ordinate elements associated with a
        DesignElement of category Reuse, or design features such as weld points associated with a Design Control
        Element), then these sub elements are returned as modified objects in the 'ServiceData'.
        
        
        Use cases:
        Use case 1:
        Assign an effectivity condition to a design feature (Cpd0DesignFeature) in a Collaborative Design model
        (Cpd0CollaborativeDesign).
        - Two alternative weld points exist in the context of a common Design Control Element (DCE).
        - Effectivity Unit=1..9 is assigned to the first weld point using operation 'setEffectivityConditions'.
        - Effectivity Unit=10..UP  is assigned to the second weld point using the same service operation.
        - Then effectivity Unit=5..15 is assigned to the DCE (Cpd0DesignControlElement), which reduces the effectivity
        range of all elements controlled by this DCE. The result is that the first weld point is effective for
        Unit=5..9, while the second is effective Unit=10..15.
        - Finally a NULL effectivity range is assigned to the DCE, which removes the effectivity condition from the
        DCE, and extends the effectivity range of all elements controlled by this DCE to their original values. The
        result is that the first weld point is again effective for Unit=1..9, while the second is effective Unit=10..UP.
        
        
        
        Use case 2:
        Assign an effectivity condition to a Design Element (DE) such that its effectivity range covers the combined
        range of effectivity of multiple other DEs in a Collaborative Design (CD) model.
        - Multiple alternative radar systems and radar screens exist in a CD (Cpd0CollaborativeDesign). Radar system R1
        is effective for Unit=1..10, while radar system R2 is effective for Unit=11..UP. Radar display unit D1 is
        effective for Unit=1..5, while radar display unit D2 is effective for Unit=6..UP. I.e. every product unit has
        exactly one radar system and one radar screen, but the combination of radar systems and screens varies for
        different product units.
        - The same radar cable C1 connects all radar units with their display screens. C1 is effective for Unit=1..UP.
        - Effective Unit=3..UP a new radar cable C2 shall replace C1 such that it covers the combined effectivity range
        for R1, R2, D1, and D2.
        - To achieve this goal, operation setEffectivityConditions is used to assign the combined effectivity range of
        R1, R2, D1, and D2, to the new radar cable C2 with affectedObjects={C2}, sampleObjects={R1, R2, D1, D2},
        opCode=11 (OR), and actionCode=1 (OVERWRITE). C2s effectivity range is now Unit=1..UP, while its effectivity
        configuration formula is Unit=1..10 or Unit=11..UP or Unit=1..5 or Unit=6..UP.
        - C2s design is completed and reviewed in this state.
        - Finally operation 'setEffectivityConditions' is used with affectedObjects={C2}, replacedObjects={C1},
        formulae={Unit>=3}, opCode=11 (OR), and actionCode=4 (REDUCE). C2s effectivity range is now Unit=3..UP, while
        C1s effectivity range is now Unit=1..2.
        
        
        
        Use case 3:
        Extend the effectivity range for a set of Design Elements (DE) such that their effectivity range also covers
        the combined effectivity configuration criteria attached to two RevisionRules.
        - Multiple alternative radar systems and radar screens exist in a CD (Cpd0CollaborativeDesign). Radar system R1
        is effective for Unit={1, 3}, while radar system R2 is effective for Unit={2, 4}. Radar display unit D1 is
        effective for Unit={1, 2}, while radar display unit D2 is effective for Unit={3, 4}. I.e. all product units
        1..4 have exactly one radar system and one radar screen, but the combination of radar systems and screens
        varies for different product units.
        - Field tests have shown that the combination between R1 and D2 is best. Therefore this combination shall be
        made effective for the upcoming new product units 5 and 6, represented by RevisionRules Rule1:Unit=5 and
        Rule2:Unit=6.
        - To achieve this goal, operation setEffectivityConditions is used to assign the combined effectivity
        configuration criteria associated to Rule1 and Rule2, to affectedObjects={R1,D2} by using sampleObjects={Rule1,
        Rule2 }, opCode=11 (OR), and actionCode=2 (EXTEND). The result is that R1s effectivity range is now Unit={1, 3,
        5, 6}, while D2s effectivity range is now Unit={3, 4, 5, 6}.
        
        """
        return cls.execute_soa_method(
            method_name='setEffectivityConditions',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'sampleObjects': sampleObjects, 'formulae': formulae, 'expressions': expressions, 'opCode': opCode, 'actionCode': actionCode, 'affectedObjects': affectedObjects, 'replacedObjects': replacedObjects, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=EffectivityConditionResponse,
        )

    @classmethod
    def setRevRuleEffectivityCriteria(cls, sampleObjects: List[POM_object], formulae: List[ConfigFormula], createPrivateCopy: bool, affectedRevRules: List[RevisionRule], productName: str, productNameSpace: str, configuratorURL: str, expressions: List[ConfigExpression], opCode: int, substituteDependentVariables: int, applyConstraints: int, applyDefault: int, actionCode: int, solveType: int, saveNow: bool) -> RevRuleEffectivityCriteriaResponse:
        """
        This operation assigns the specified effectivity criteria to the revision rules listed in 'affectedRevRules'.
        If a ('productName','productNameSpace') combination is specified, then existing validation records associated
        with this product will be added to the response. If a non zero value is specified for parameters
        'substituteDependentVariables', 'applyConstraints', and/or 'applyDefaults', then corresponding effectivity
        configurator rules will be applied to the effectivity criteria and the corresponding validation records will be
        returned in the response structure. Runtime copies of the RevisionRules listed in 'affectedRevRules' are
        created with the requested effectivity configuration criteria if parameter 'createPrivateCopy' is set to true.
        In that case the response structure will reference the new runtime copies. Otherwise WRITE or REVISE privileges
        for 'affectedRevRules' are required. If parameter saveNow is set to true and 'createPrivateCopy' is set to
        false, then RevisionRule modifications are saved. Otherwise, 'affectedRevRules' are updated, but not saved.
        
        This operation connects to the effectivity configurator service whose service endpoint is specified by the
        configuratorURL parameter. The actual conversion to and from formula strings is performed by this configurator
        service. The result may vary depending on the choice of service endpoint. For example a given configurator
        might return shorthand representations for formulae if these are unique in the context of the specified
        product. Teamcenter 9 only supports local built in Teamcenter configurators where parameter configuratorURL can
        be set to an empty string. Teamcenter configurators used in the EffectivityManagement service interface encode
        configuration formulae in Explicit Teamcenter Language for which no product context is required (see operation
        'getEffectivityExpressions' for more details on Explicit Teamcenter Language  and 'productName' and
        'productNameSpace').
        
        Parameter solveType can be used to specify a filter strategy when evalutating the effectivity configuration
        criteria. This parameter is used to combine one or more of the following with binary OR:
        
        - MISMATCH                    1: objects conflicting with the solve criteria
        - EXPLICIT                        2: objects explicitly satisfying the solve criteria
        - COPRIME                        4: objects potentially satisfying the solve criteria
        - TRUE                            8: objects having an effectivity condition = TRUE
        - FALSE                            16: objects having an effectivity condition = FALSE
        - CONDITION                    32: objects having a non constant effectivity condition
        - ERROR_CHECK                64: objects having effectivity conditions returning an error
        - NO_CONDITION            128: objects with configurable behavior without a condition
        - NO_CONFIG_BEHAVIOR    256: objects without configurable behavior
        - INVERT                        512: inverts the filter results
        
        
        
        For example a solveType value of 398=256+128+8+4+2 produces the same filter results as 529= 512+16+1, but is
        more efficient if most objects are expected to fail the filter. On the other hand if most objects are expected
        to pass the filter than 529 is more efficient. If in doubt a value of 529 is recommended in most cases.
        
        Use cases:
        Setup a RevisionRule to define effectivity configuration criteria that cover the combined effectivity ranges of
        a set of Design Elements (DE) in a Collaborative Design (CD) model.
        
        - Multiple alternative radar systems and radar screens exist in a CD (Cpd0CollaborativeDesign). Radar system R1
        is effective for Unit={1, 3}, while radar system R2 is effective for Unit={2, 4}. Radar display unit D1 is
        effective for Unit={1, 2}, while radar display unit D2 is effective for Unit={3, 4}.
        - A radar cable C1 shall be added to the CD suitable to connect any radar system with any radar screen. To find
        DEs that can exist together with any radar system and screen a RevisionRule shall be defined that covers their
        combined effectivity range.
        - To achieve this goal, operation setRevRuleEffectivityCriteria is used to assign the combined effectivity
        range for R1, R2, D1, and D2, to a transient RevisionRule copy of the system RevisionRule Rule1 Working; Any
        Status.
        - affectedRevRules={Rule1} 
        - sampleObjects={R1, R2, D1, D2}
        - opCode=11 (OR)
        - actionCode=1 (SET)
        - substituteDependentVariables=0 (DISABLE)
        - applyConstraints=1 (ENABLE)
        - applyDefault=0 (DISABE)
        - solveType=529 (MISMATCH|FALSE|INVERT)
        - saveNow=false
        - createPrivateCopy=true
        - The result is that a new transient RevisionRule Rule1 is created with effectivity configuration criteria
        Unit=1..4.
        
        """
        return cls.execute_soa_method(
            method_name='setRevRuleEffectivityCriteria',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'sampleObjects': sampleObjects, 'formulae': formulae, 'createPrivateCopy': createPrivateCopy, 'affectedRevRules': affectedRevRules, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL, 'expressions': expressions, 'opCode': opCode, 'substituteDependentVariables': substituteDependentVariables, 'applyConstraints': applyConstraints, 'applyDefault': applyDefault, 'actionCode': actionCode, 'solveType': solveType, 'saveNow': saveNow},
            response_cls=RevRuleEffectivityCriteriaResponse,
        )

    @classmethod
    def getAvailableProductEffectivity(cls, revisionRule: RevisionRule, exprsToTest: List[ConfigExpression], familiesToTest: List[ConfigurationFamily], substituteDependentVariables: int, applyConstraints: int, productName: str, productNameSpace: str, configuratorURL: str) -> AvailableProductEffectivityResponse:
        """
        This operation returns available effectivity that could be used to satisfy the specified RevisionRule, or the
        subset of the RevisionRule as defined by parameters 'exprsToTest' and 'familiesToTest'. Parameters
        'productName' and 'productNameSpace' can be used to identify a product whose effectivity constraints will be
        considered in this operation. No product effectivity constraints are considered if these parameters are empty.
        If parameters 'exprsToTest' and 'familiesToTest' are empty and no product context is given the unmodified basic
        range of effectivity represented by the given RevisionRule is returned. If 'exprsToTest' or 'familiesToTest'
        are provided, then the available range of effectivity for these expressions and /or families will be returned.
        
        The operation forms a config expression from the provided input parameters by ORing the expressions in
        parameter 'exprsToTest' (resulting in TRUE if 'exprsToTest' was empty), and ANDing the result with the
        effectivity associated with the RevisionRule (using TRUE if none was specified). The response will return
        available effectivity for the union set of families used in this config expression with the families listed in
        'familiesToTest'. If the resulting list is empty, available effectivity for all families will be returned.
        Effectivity is considered available if it neither conflicts with the config expression nor with the effectivity
        validation rules defined in the product identified by parameters 'productName' and 'productNameSpace' (if
        present). Because of the way the config expression is formed the operation can only return meaningful results
        if either a RevisionRule, or a list of 'exprsToTest', or a list of 'familiesToTest' (or a combination of the
        above) are provided. If all parameters are NULL the response will return no effectivity.
        
        Operation 'getRevRuleEffectivityCriteria' should be used if symbolic variable substitution and/or constraint
        processing is required for *all* variant option families, or if validation records are required that specify
        violated constraints.
        
        Operation 'setEffectivityConditions' should be used if assigning the combined effectivity range covering a set
        of existing objects does not require the evaluation of configurator constraints. The difference is that
        'getAvailableProductEffectivity' supports parameters that enable effectivity configurator validation rules,
        which can be used to eliminate effectivity combinations from the combined range that are not available
        according to the current set of validation rules. Eliminating invalid combinations reduces the number of false
        positive results detected by automated collision detection systems such as Teamcenter Clearance Analysis. On
        the other hand persisting such expressions causes the condition to be out of date if the set of configurator
        validation rules changes.
        This operation connects to the effectivity configurator service whose service endpoint is specified by the
        'configuratorURL' parameter. The actual evaluation of configurator rules to trim available effectivity, and
        conversions to and from formula strings, are performed by this configurator service. The resulting formula
        string conversions may vary depending on the choice of service endpoint. For example a given configurator might
        return shorthand representations for formulae if these are unique in the context of the specified product.
        Teamcenter 9 only supports local builtin Teamcenter configurators where parameter 'configuratorURL' can be set
        to an empty string. Teamcenter configurators used in the 'EffectivityManagement' service interface encode
        configuration formulae in Explicit Teamcenter Language for which no product context is required (see operation
        'getEffectivityExpressions' for more details on Explicit Teamcenter Language and 'productName' and
        'productNameSpace').
        
        Use cases:
        Use case 1:Initialize an effectivity criteria dialog for a given RevisionRule.
        - The application uses operation 'getAvailableProductEffectivity' with 'productName=, productNameSpace=,
        exprsToTest={}, and familiesToTest={}, substituteDependentVariables=0, applyConstraints=0.'
        - The response will contain effectivity ranges as they are stored on the RevisionRule without additional
        configurator processing. References to effectivity configuration families (ConfigurationFamily) are returned in
        'ServiceData'.
        
        
        
        Use case 2:
        Initialize an effectivity criteria dialog for a product that was created using the Teamcenter effectivity
        configurator with the intent to setup a new RevisionRule.
        - The application uses operation 'getAvailableProductEffectivity' with 'productName=MyProductItemID,
        productNameSpace=MyProductRevID, exprsToTest={}, and familiesToTest={}, substituteDependentVariables=0,
        applyConstraints=0.'
        - The response will contain all nominal effectivity ranges in this product. No constraint processing is
        required. References to effectivity configuration families (ConfigurationFamily) are returned in ServiceData.
        
        
        Use case 3:
        Initialize an effectivity criteria dialog for a product that is associated with a Collaborative Design (CD)
        model with the intent to setup a new RevisionRule.
        - Identifiers for the product associated with the CD (Cpd0CollaborativeDesign) are obtained from CD properties
        'mdl0config_product_name', and 'mdl0config_prod_namespace'.
        - The application uses operation 'getAvailableProductEffectivity' with 'productName=' and 'productNameSpace= '
        as obtained in the previous step,' exprsToTest={}, and familiesToTest={}, substituteDependentVariables=0,
        applyConstraints=0.'
        - The response will contain all nominal effectivity ranges in this product. No constraint processing is
        required. References to effectivity configuration families (ConfigurationFamily) are returned in 'ServiceData'.
        
        
        Use case 4:
        Iterate through all available configuration families (ConfigurationFamily) with the intent to assign values to
        each configuration family in an iterative process while dynamically requesting the remaining available range of
        values for a given configuration family.
        - The application uses operation 'getAvailableProductEffectivity' with the same values for 'productName', and
        'productNameSpace' that was used in the initialization step (see above use cases). Parameter exprsToTest is
        used to accumulate the choices that were made in previous iteration steps. Parameter
        'familiesToTest={nextFamilyInList} 'is used to specify the family for which available value ranges are
        requested in this iteration step. The remaining parameter values are 'substituteDependentVariables'=0, and
        'applyConstraints'=1.
        - The response will contain the available value range for the specified 'familiesToTest'.
        
        
        
        Use case 5:
        Request the effectivity configuration criteria in which at least one of multiple objects configures and
        eliminate criteria from this result that violate any current validation rules in the context of a Collaborative
        Design (CD) model.
        - A radar cable design shall be given an effectivity condition such that it configures whenever any of a given
        set of radar systems and radar screens configures.
        - Identifiers for the product associated with the CD (Cpd0CollaborativeDesign) are obtained from CD  properties
        'mdl0config_product_name', and 'mdl0config_prod_namespace'.
        - The application uses operation 'getEffectivityConditions' to obtain the set of conditions associated with the
        radar systems and screens.
        - The application uses operation 'getAvailableProductEffectivity' where parameter 'exprsToTest' is used to pass
        the effectivity conditions associated with the radar systems and screens. Parameter familiesToTest is left
        empty. The remaining parameter values are 'substituteDependentVariables'=0, and 'applyConstraints'=1.
        - The response will contain the actually available effectivity range for the listed radar systems and screens
        according to the current set of effectivity configurator validation rules.
        
        """
        return cls.execute_soa_method(
            method_name='getAvailableProductEffectivity',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'revisionRule': revisionRule, 'exprsToTest': exprsToTest, 'familiesToTest': familiesToTest, 'substituteDependentVariables': substituteDependentVariables, 'applyConstraints': applyConstraints, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=AvailableProductEffectivityResponse,
        )

    @classmethod
    def getConfigurableProducts(cls, configuratorURL: str) -> ConfigurableProductsResponse:
        """
        This operation returns all effectivity configurable products from the effectivity configurator identified by
        parameter 'configuratorURL'. The operation connects to the effectivity configurator service whose service
        endpoint is specified by the 'configuratorURL' parameter. The query for available effectivity configurable
        products is performed by this configurator service. Teamcenter 9 only supports local builtin Teamcenter
        configurators where parameter 'configuratorURL' can be set to an empty string. Teamcenter configurators use an
        Item ID for 'productName' and an Item Revision ID or RDV  Product Context Identifier for 'productNameSpace'.
        Teamcenter configurators return all ItemRevision objects that allocate effectivity configuration families as
        effectivity configurable products.
        
        Use cases:
        Associate a product item with a Collaborative Design (CD) model so that effectivity configurator rules
        associated with this product are also associated with the CD. The assumption is that the CD is an application
        model (Mdl0ApplicationModel) for the product represented by this product item.
        - Operation 'getConfigurableProducts' is used to obtain a list of all effectivity configurable products.
        - User reviews the list of available ('productName, productNameSpace') and selects one.
        - Operation 'findWorkspaceObjects' is used to locate the product ItemRevision that corresponds to the given
        ('productName, productNameSpace') tuple.
        - Operation 'createRelations' is used to attach the product ItemRevision to the CD using 'primaryObject=CD,
        secondaryObject=MyProductItemRevision, and relationType=Mdl0HasConfiguratorContext.'
        - Operation 'refreshObjects' is used to refresh the CD.
        - Identifiers for the product associated with the CD (Cpd0CollaborativeDesign) are obtained from CD properties
        'mdl0config_product_name', and 'mdl0config_prod_namespace.'
        
        """
        return cls.execute_soa_method(
            method_name='getConfigurableProducts',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'configuratorURL': configuratorURL},
            response_cls=ConfigurableProductsResponse,
        )

    @classmethod
    def getEffectivityConditions(cls, affectedObjects: List[POM_object], configuratorURL: str) -> EffectivityFormulaeResponse:
        """
        This operation returns the effectivity conditions associated with the objects specified in 'affectedObjects' in
        form of a configurator specific formula string. This operation connects to the effectivity configurator service
        whose service endpoint is specified by the 'configuratorURL' parameter. The actual conversion to formula
        strings is performed by this configurator service. The result may vary depending on the choice of service
        endpoint. For example a given configurator might return shorthand representations for formulae if these are
        unique in the context of the specified product. Teamcenter 9 only supports local built in Teamcenter
        configurators where parameter 'configuratorURL' can be set to an empty string.
        
        Use cases:
        Obtain a formula string representation for the effectivity configuration conditions associated with set of
        design elements (Cpd0DesignModelElement) in a Collaborative Design model (Cpd0CollaborativeDesign).
        - Multiple weld points (Cpd0DesignFeature) exist in the context of a Design Control Element
        (Cpd0DesignControlElement).
        - The effectivity range for each weld point is therefore equivalent to the intersection between their own
        effectivity range and the range of effectivity for the Design Control Element (DCE).
        - In order to review the effectivity ranges an application requests the effectivity condition for each weld
        point using operation 'getEffectivityConditions'.
        
        """
        return cls.execute_soa_method(
            method_name='getEffectivityConditions',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'affectedObjects': affectedObjects, 'configuratorURL': configuratorURL},
            response_cls=EffectivityFormulaeResponse,
        )

    @classmethod
    def getEffectivityExpressions(cls, formulae: List[ConfigFormula], productName: str, productNameSpace: str, configuratorURL: str) -> EffectivityExpressionsResponse:
        """
        This operation returns effectivity expressions for a given set of effectivity formulae in the context of the
        given productName and productNameSpace combination. This operation connects to the effectivity configurator
        service whose service endpoint is specified by the configuratorURL parameter. Any conversion from formula
        strings is performed by this configurator service. Depending on the (configurator specific) encoding the
        operation might require the specification of a product context using parameters productName and
        productNameSpace. For example, a given configurator might recognize shorthand representations for formulae if
        these are unique in the context of the specified product. Teamcenter 9 only supports local built in Teamcenter
        configurators where parameter configuratorURL can be set to an empty string. Teamcenter configurators dont
        require a product context if the formulae are in Explicit Teamcenter Language. The encoding is explicit if all
        lexemes are uniquely identified, e.g. [OptionNamespace]FamilyName = UniqueValue, where no product context is
        required to determine the family name for a value, or the option namespace for the family. A variant formula is
        in Explicit Teamcenter Language if its form is explicit and comprised of the lexemes documented for the
        Teamcenter Variant Formula property. Teamcenter configurators support shorthand encodings like FamilyName =
        UniqueValue or UniqueValue if the lexemes used in the shorthand encoding are unique in the specified product
        context.
        
        Use cases:
        Obtain an effectivity expression (ConfigExpression) for a formula string representation (ConfigFormula) in the
        context of a Collaborative Design model (Cpd0CollaborativeDesign).
        Some effectivity SOA operations, e.g. 'getEffectivityConditions', formula string representations, which might
        be easier to review if presented in expression format.
        - A weld point (Cpd0DesignFeature) exists in the context of a Design Control Element (Cpd0DesignControlElement).
        - Effectivity Unit=10..UP is assigned to the weld point using operation 'setEffectivityConditions'.
        - The same operation is then used to assign effectivity Unit=1..5 to Design Control Element (DCE), which
        reduces the effectivity range of all model elements controlled by this DCE. The result is that the weld point
        above is no longer effective since its effectivity range has no overlap with the DCE that controls it.
        - In order to understand why the weld point is not effective an application requests the effectivity condition
        for the weld point using operation 'getEffectivityConditions'.
        - Depending on the number of control elements associated with this weld point the effectivity condition formula
        in the response might be difficult to comprehend. Therefore the application requests a conversion into an
        effectivity expression using operation 'getEffectivityExpressions'.
        
        """
        return cls.execute_soa_method(
            method_name='getEffectivityExpressions',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'formulae': formulae, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=EffectivityExpressionsResponse,
        )

    @classmethod
    def getEffectivityTables(cls, formulae: List[ConfigFormula], productName: str, productNameSpace: str, configuratorURL: str) -> EffectivityTablesResponse:
        """
        This operation returns effectivity table representations in form of rows consisting of 
        (effect in,effect out) tuples for a given set of effectivity condition formulae. This operation connects to the
        effectivity configurator service whose service endpoint is specified by the configuratorURL parameter. Any
        conversion from formula strings is performed by this configurator service. Depending on the (configurator
        specific) encoding the operation might require the specification of a product context using parameters
        productName and productNameSpace. For example, a given configurator might recognize shorthand representations
        for formulae if these are unique in the context of the specified product. Teamcenter 9 only supports local
        built in Teamcenter configurators where parameter configuratorURL can be set to an empty string. Teamcenter
        configurators dont require a product context if the formulae are in Explicit Teamcenter Language. The encoding
        is explicit if all lexemes are uniquely identified, e.g. [OptionNamespace]FamilyName = UniqueValue, where no
        product context is required to determine the family name for a value, or the option namespace for the family. A
        variant formula is in Explicit Teamcenter Language if its form is explicit and comprised of the lexemes
        documented for the Teamcenter Variant Formula property. Teamcenter configurators support shorthand encodings
        like FamilyName = UniqueValue or UniqueValue  if the lexemes used in the shorthand encoding are unique in the
        specified product context.
        
        Use cases:
        Obtain an effectivity table (EffectivityTable) for a formula string representation (ConfigFormula) in the
        context of a Collaborative Design model (Cpd0CollaborativeDesign).
        Some effectivity SOA operations, e.g. 'getEffectivityConditions', return formula string representations, which
        might be easier to review if presented in a table of rows consisting of (effect in,effect out) tuples.
        - Two weld points (Cpd0DesignFeature) exists, each in the context of multiple different Design Control Elements
        (Cpd0DesignControlElement).
        - The effectivity range for each weld point is therefore equivalent to the intersection between their own
        effectivity range and the combined range of effectivity for their respective set of Design Control Elements
        (DCE).
        - In order to compare the effectivity ranges for the 2 weld points an application requests the effectivity
        condition for each weld point using operation 'getEffectivityConditions'.
        - Depending on the number of control elements associated with each weld point the effectivity condition formula
        in the response can be different even if their ranges are logically equivalent. Therefore the application
        requests a conversion to effectivity tables using operation 'getEffectivityTables' so that the two effectivity
        ranges can be compared.
        
        """
        return cls.execute_soa_method(
            method_name='getEffectivityTables',
            library='ConfigFilterCriteria',
            service_date='2011_06',
            service_name='EffectivityManagement',
            params={'formulae': formulae, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=EffectivityTablesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2013_05.StructureExpansionLite import PartialExpansionControls, LineAndPaths, Controls
from tcsoa.gen.Internal.StructureManagement._2013_12.StructureExpansionLite import ExpansionResponse2
from typing import List
from tcsoa.gen.BusinessObjects import RuntimeBusinessObject
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class StructureExpansionLiteService(TcService):

    @classmethod
    def getUndelivered2(cls, undeliveredLineUids: List[str], controls: Controls) -> ExpansionResponse2:
        """
        This operation returns Fnd0BOMLineLite as objects for input UIDs of Fnd0BOMLineLite. 
        It may be required to invoke this operation more than once as the number of Fnd0BOMLineLite returned depends on
        'outputPageSize' specified in 'Controls'. The 'RelationAndTypesCriteria' of 'Controls' is used to retrieve
        Dataset, named references objects with FMS tickets when named reference object is file.
        
        
        Use cases:
        This operation helps while working with large structure. 
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets VariantRule on BOMWindow.
        3. Client invokes 'expandNext2' with top BOMLine to get next set of configured children as Fnd0BOMLineLite
        objects.
        4. Client checks if there are any undelivered Fnd0BOMLineLite objects in 'ExpansionResponse2'
        5. Client invokes 'getUndelivered2' operation to get next set of undelivered Fnd0BOMLineLite(s) along with
        Dataset.
        6. Client uses configured structure and Dataset to render the product.
        7. Client closes the BOMWindow to release resources.
        """
        return cls.execute_soa_method(
            method_name='getUndelivered2',
            library='Internal-StructureManagement',
            service_date='2013_12',
            service_name='StructureExpansionLite',
            params={'undeliveredLineUids': undeliveredLineUids, 'controls': controls},
            response_cls=ExpansionResponse2,
        )

    @classmethod
    def unloadBelow2(cls, lines: List[RuntimeBusinessObject]) -> ServiceData:
        """
        This operation destroys Fnd0BOMLineLite children lines of input Fnd0BOMLineLite or BOMLine parent lines from
        Teamcenter server.
        
        Use cases:
        This operation is useful to release resources after rendering product.
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets VariantRule on BOMWindow.
        3. Client invokes 'expandNext2' with top BOMLine to get next set of configured children as Fnd0BOMLineLite
        objects.
        4. Client performs expansion.
        5. Client uses configured structure and Dataset to render the product.
        6. Client releases resource on Teamcenter server by calling unload operation.
        """
        return cls.execute_soa_method(
            method_name='unloadBelow2',
            library='Internal-StructureManagement',
            service_date='2013_12',
            service_name='StructureExpansionLite',
            params={'lines': lines},
            response_cls=ServiceData,
        )

    @classmethod
    def expandBasedOnOccurrenceList2(cls, linePaths: List[LineAndPaths], controls: PartialExpansionControls) -> ExpansionResponse2:
        """
        The operation performs partial expansion for the given 'LineAndPaths'. This operation returns Dataset(s), named
        reference objects with FMS tickets which are requested through 'RelationAndTypesCriteria' in
        'PartialExpansionControls' for the partially expanded child Fnd0BOMLineLite(s).
        Note that partial expansion for ImanItemBOPLine is not supported.
        
        
        Use cases:
        This operation is suited to get partially expanded configured structure and related Dataset for read-only
        scenarios, e.g. Visualization. 
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets VariantRule on BOMWindow.
        3. Client pulls the 'occurrence_list' from the VisStructureContext object.
        3. Client invokes 'expandBasedOnOccurrenceList2' to get partially expanded configured structure.
        4. Client uses configured structure and Dataset to render the product.
        5. Client closes the BOMWindow to release resources.
        """
        return cls.execute_soa_method(
            method_name='expandBasedOnOccurrenceList2',
            library='Internal-StructureManagement',
            service_date='2013_12',
            service_name='StructureExpansionLite',
            params={'linePaths': linePaths, 'controls': controls},
            response_cls=ExpansionResponse2,
        )

    @classmethod
    def expandNext2(cls, linesToExpand: List[RuntimeBusinessObject], controls: Controls) -> ExpansionResponse2:
        """
        The operation performs top down expansion for the given parent Fnd0BOMLineLite or BOMLine. The level of
        expansion is based upon input 'Controls' such as maxLevel. The 'RelationAndTypesCriteria' of 'Controls' is used
        to retrieve Dataset, named references objects with FMS tickets when named reference object is file.
        
        Use cases:
        This operation is suited to get configured structure and related Dataset for read-only scenarios, e.g.
        Visualization. 
        
        1. Client creates BOMWindow with RevisionRule and sets top BOMLine.
        2. Client optionally sets Variant Rule on BOMWindow.
        3. Client invokes 'expandNext2' with top BOMLine to get next set of configured children as Fnd0BOMLineLite
        objects.
        4. Client invokes 'expandNext2' recursively till structure is fully expanded.
        5. Client uses configured structure and Dataset to render the product.
        6. Client closes the BOMWindow to release resources.
        """
        return cls.execute_soa_method(
            method_name='expandNext2',
            library='Internal-StructureManagement',
            service_date='2013_12',
            service_name='StructureExpansionLite',
            params={'linesToExpand': linesToExpand, 'controls': controls},
            response_cls=ExpansionResponse2,
        )



from tcsoa.gen.Internal_Rdv._2009_01.services import VariantManagementService as imp0
from tcsoa.gen.Internal_Rdv._2010_04.services import VariantManagementService as imp1
from tcsoa.gen.Internal_Rdv._2009_04.services import VariantManagementService as imp2
from tcsoa.gen.Internal_Rdv._2009_05.services import VariantManagementService as imp3
from tcsoa.gen.Internal_Rdv._2008_03.services import VariantManagementService as imp4
from tcsoa.gen.Internal_Rdv._2007_06.services import VariantManagementService as imp5
from tcsoa.gen.Internal_Rdv._2007_09.services import VariantManagementService as imp6
from tcsoa.base import TcService


class VariantManagementService(TcService):
    addDesignToProduct = imp0.addDesignToProduct
    addPartToProduct = imp1.addPartToProduct
    applyNVEMetaExpression = imp0.applyNVEMetaExpression
    askNVEMetaExpression = imp0.askNVEMetaExpression
    createMultipleNamedVariantExpressions = imp2.createMultipleNamedVariantExpressions
    createSavedVariantRules = imp2.createSavedVariantRules
    deleteMultipleNamedVariantExpressions = imp2.deleteMultipleNamedVariantExpressions
    executeAdhocSearchWithOverlays = imp3.executeAdhocSearchWithOverlays
    executeSearchWithOverlays = imp3.executeSearchWithOverlays
    getApnComponents = imp0.getApnComponents
    getArchbreakdownBomlineChildComponents = imp0.getArchbreakdownBomlineChildComponents
    getArchbreakdownMeapnChildComponents = imp0.getArchbreakdownMeapnChildComponents
    getValidBackgroundOverlays = imp1.getValidBackgroundOverlays
    getValidoverlayBomlinesInfo = imp4.getValidoverlayBomlinesInfo
    getVariabilityInfo = imp5.getVariabilityInfo
    getVariantExprXOChartData = imp6.getVariantExprXOChartData
    realignNVEMetaExpressionTokens = imp3.realignNVEMetaExpressionTokens
    reapplyNVEMetaExpressions = imp0.reapplyNVEMetaExpressions
    replaceDesignInProduct = imp0.replaceDesignInProduct
    replacePartInProduct = imp1.replacePartInProduct
    validateNVEMetaExpressions = imp0.validateNVEMetaExpressions



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ChangeTypeInfo(TcBaseObj):
    """
    Structure holds the Change type information.
    
    :var typeName: Name of the Change type.
    :var typeDisplayName: Display name of the Change type.
    """
    typeName: str = ''
    typeDisplayName: str = ''


@dataclass
class ContextDataInput(TcBaseObj):
    """
    The parameters required to find the creatable ChangeItem type.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var object: Context data for creating ChangeItem types. If this input is not specified then the allowed Change
    types will be determined based only on the conditions specified for Change type creation in BMIDE. This input is
    used to determine the Change types that are allowed to be created for creating derive Change objects.
    Note: The calling client is responsible for converting the inputs to a string using the appropriate function(s) in
    the SOA client framework Property class (i.e. Property.toDateString).
    :var baseTypeName: Base type name. If this input is specified then all the Change types in this type hierarchy will
    be evaluated. If not specified then all the change types in ChangeItem hierarchy will be evaluated for determining
    the allowed change types. Valid value for this input is ChangeItem or its sub type.
    :var exclusionTypeNames: A list of Change types to be excluded from the returned list. If this input is specified
    then Change types specified in this list will be excluded while determining the allowed Change types. Valid value
    for this input is ChangeItem or its sub type.
    """
    clientId: str = ''
    object: str = ''
    baseTypeName: str = ''
    exclusionTypeNames: List[str] = ()


@dataclass
class CreatableChangeTypesOut(TcBaseObj):
    """
    This is output of getCreatableChangeTypes operation.
    
    :var clientId: The unmodified value from the ContextDataInput.clientId. This can be used by the caller to indentify
    this data structure with the source input data.
    :var allowedChangeTypes: A list of ChangeTypeInfo representing Change types that are allowed to be created.
    """
    clientId: str = ''
    allowedChangeTypes: List[ChangeTypeInfo] = ()


@dataclass
class CreatableChangeTypesResponse(TcBaseObj):
    """
    This is response Structure of getCreatableChangeTypes operation.
    
    :var output: A list of CreatableChangeTypesOut objects representing ChangeTypeInfo objects.
    :var serviceData: Service data including partial errors that are mapped to the client id.
    """
    output: List[CreatableChangeTypesOut] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, TC_Project
from tcsoa.gen.StructureManagement._2008_06.Structure import DefaultNaming
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class NewItemInfo2(TcBaseObj):
    """
    The proposed new ItemIds and new ItemNames.
    
    :var newItemId: The proposed new ItemId
    :var newItemName: The proposed new ItemName
    """
    newItemId: str = ''
    newItemName: str = ''


@dataclass
class ProjectInfo(TcBaseObj):
    """
    Project Information for Structure Clone
    
    :var assign: Flag that indicates if assigning projects
    :var validate: Flag that indicates if project needs validation
    :var projects: Vector of user selected projects
    """
    assign: bool = False
    validate: bool = False
    projects: List[TC_Project] = ()


@dataclass
class ValidateStructureItemIdsInfo2(TcBaseObj):
    """
    Input to ValidateStructureItemIds2. It contains the top BOMLine, map of oldItemId to NewItemInfo(NewItemId,
    NewItemName), default naming pattern and list of dependency types specified by the user.
    
    :var topLine: Top BOMLine or the selected bom line of the structure or sub structure to be cloned.
    :var inputMap: Map of OldItemIDs to  NewItemInfo2 that contains the proposed New ItemId and New ItemName.
    :var defaultName: Pattern to form a new ItemId.
    :var projects: List of user defined projects to which the duplicated Item objects will be added.
    :var options: List of dependency types.
    """
    topLine: BOMLine = None
    inputMap: DuplicateIdMap2 = None
    defaultName: DefaultNaming = None
    projects: ProjectInfo = None
    options: List[DependencyType2] = ()


@dataclass
class ValidateStructureItemIdsResponse2(TcBaseObj):
    """
    The response from ValidateStructureItemIds, containing a map of OldItemID to 'NewItemInfo', list of ItemRevision
    objects failed validation and a 'ServiceData' object.
    
    :var clonedIdMap: Map of OldItemID to 'NewItemInfo' that contains the proposed New ItemId and New ItemName
    :var serviceData: 'ServiceData' object.  The error stack will contain those ItemRevision objects that failed
    cloning and the reason.
    """
    clonedIdMap: DuplicateIdMap2 = None
    serviceData: ServiceData = None


@dataclass
class DuplicateInputInfo2(TcBaseObj):
    """
    Input for Duplicate2 SOA API
    
    :var topLine: The top BOMLine or selected BOMLine of the structure to be cloned.  The user can select a sub
    assembly from the original structure to clone.  This input cannot be NULL.  If it is the duplicate dialog will not
    come up at all.
    :var duplicateIdMap: A map of oldItemId and NewItemInfo structure. The newItemInfo is made up of the proposed new
    ItemId and the proposed new ItemName
    :var defaultName: The pattern to form a new ItemId
    :var projects: The user selected projects to add the duplicated Item objects to
    :var renameCadFile: Whether to rename the cad files or not
    :var options: The list of dependency types.  This defines the cad traversal.
    :var duplicateOption:  A bitmap for duplicate option. 1 means smart selection.
    """
    topLine: BOMLine = None
    duplicateIdMap: DuplicateIdMap2 = None
    defaultName: DefaultNaming = None
    projects: ProjectInfo = None
    renameCadFile: bool = False
    options: List[DependencyType2] = ()
    duplicateOption: int = 0


class DependencyType2(Enum):
    """
    Dependency Types
    """
    NoDep = 'NoDep'
    Drawing = 'Drawing'
    PartFamilyMaster = 'PartFamilyMaster'
    PartFamilyMember = 'PartFamilyMember'
    Required = 'Required'
    AllDep = 'AllDep'
    ExcludeFromBom = 'ExcludeFromBom'


"""
OldItemIds mapped to the NewItemInfo2 structure.
"""
DuplicateIdMap2 = Dict[str, NewItemInfo2]



from __future__ import annotations

from tcsoa.gen.Ai._2006_03.Ai import ApplicationRef
from tcsoa.gen.Internal.Ai._2008_06.Ai import GenerateMonolithicJtOptions, EndGenerateMonolithicJtResponse, BeginGenerateMonolithicJtResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Ai._2007_12.Ai import Configuration2
from tcsoa.base import TcService


class AiService(TcService):

    @classmethod
    def invoke(cls, methodName: str, strIn: str) -> str:
        """
        Invoke method that takes a string and outputs a string. This is in place for backward
        compatibility with existing AIWS clients that used this method to call custom hook furnctions.
        
        Exceptions:
        >Thrown on service failure.
        """
        return cls.execute_soa_method(
            method_name='invoke',
            library='Internal-Ai',
            service_date='2008_06',
            service_name='Ai',
            params={'methodName': methodName, 'strIn': strIn},
            response_cls=str,
        )

    @classmethod
    def endGenerateMonolithicJt(cls, request: BusinessObject, serverMode: int) -> EndGenerateMonolithicJtResponse:
        """
        Get the status of a TSTK request object. To be used in conjunction with BeginGenerateMonolithicJt.
        """
        return cls.execute_soa_method(
            method_name='endGenerateMonolithicJt',
            library='Internal-Ai',
            service_date='2008_06',
            service_name='Ai',
            params={'request': request, 'serverMode': serverMode},
            response_cls=EndGenerateMonolithicJtResponse,
        )

    @classmethod
    def beginGenerateMonolithicJt(cls, startingNode: ApplicationRef, itemId: str, revId: str, config: Configuration2, options: GenerateMonolithicJtOptions, serverMode: int) -> BeginGenerateMonolithicJtResponse:
        """
        Generates a monolithic jt file given a structure.
        """
        return cls.execute_soa_method(
            method_name='beginGenerateMonolithicJt',
            library='Internal-Ai',
            service_date='2008_06',
            service_name='Ai',
            params={'startingNode': startingNode, 'itemId': itemId, 'revId': revId, 'config': config, 'options': options, 'serverMode': serverMode},
            response_cls=BeginGenerateMonolithicJtResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TCCalendarEvent, TCCalendar, Schedule
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ModifyCalendarEvent(TcBaseObj):
    """
    The information necessary to modify a calendar event.
    
    :var event: The event to modify
    :var eventAttributes: The new event data
    """
    event: TCCalendarEvent = None
    eventAttributes: CalendarEventData = None


@dataclass
class RangeData(TcBaseObj):
    """
    A single time range.  The startRange and endRange represent minutes since midnight.
    There is a special range (startRange=480 and endRange=480) which represents a non-working range.
    
    :var startRange: The starting offset.
    :var endRange: The ending offset.
    """
    startRange: int = 0
    endRange: int = 0


@dataclass
class CalendarContainer(TcBaseObj):
    """
    A container to hold the Calendar Events related to CalendarData
    
    :var tccalendarAttributes: The Calendar information
    :var newEvents: The event information
    """
    tccalendarAttributes: CalendarData = None
    newEvents: List[CalendarEventData] = ()


@dataclass
class CalendarData(TcBaseObj):
    """
    The information necessary to create a single TCCalendar including daily default ranges.
    
    :var name: The name of the calendar
    :var description: The description of the calendar
    :var thuRanges: The ranges for Thursday
    :var friRanges: The ranges for Friday
    :var satRanges: The ranges for Saturday
    :var schedule: The schedule this calendar relates to (or null)
    :var source: The resource this calendar relates to (or null)
    :var baseCalendar: The parent BASE calendar (or null)
    :var type: The type of calendar
    :var sunRanges: The ranges for Sunday
    :var monRanges: The ranges for Monday
    :var tueRanges: The ranges for Tuesday
    :var wedRanges: The ranges for Wednesday
    """
    name: str = ''
    description: str = ''
    thuRanges: List[RangeData] = ()
    friRanges: List[RangeData] = ()
    satRanges: List[RangeData] = ()
    schedule: Schedule = None
    source: BusinessObject = None
    baseCalendar: TCCalendar = None
    type: int = 0
    sunRanges: List[RangeData] = ()
    monRanges: List[RangeData] = ()
    tueRanges: List[RangeData] = ()
    wedRanges: List[RangeData] = ()


@dataclass
class CalendarEventData(TcBaseObj):
    """
    The information for a single CalendarEvent.
    
    :var firstRecurStart: The date of the event
    :var firstRecurEnd: Placeholder for recurring end date. (not currently used)
    :var eventExpiryDate: Placeholder for recurring event expiration. (not currently used)
    :var eventRanges: The time ranges for the event
    :var eventType: Placeholder for event type. (not currently used)
    :var numRecurrences: Placeholder number of recurrences. (not currently used)
    :var recurInterval: Placeholder for the recurring interval. (not currently used)
    :var recurDaysOfWeek: Placeholder for mask of days. (not currently used)
    :var recurWeeksOfMonth: Placeholder for mask of weeks. (not currently used)
    :var recurMonth: Placeholder for mask of months. (not currently used)
    """
    firstRecurStart: datetime = None
    firstRecurEnd: datetime = None
    eventExpiryDate: datetime = None
    eventRanges: List[RangeData] = ()
    eventType: int = 0
    numRecurrences: int = 0
    recurInterval: int = 0
    recurDaysOfWeek: int = 0
    recurWeeksOfMonth: int = 0
    recurMonth: int = 0


@dataclass
class CalendarResponse(TcBaseObj):
    """
    The response containing a created calendars and the events.
    
    :var calendar: The calendar
    :var calendarEvent: The events related to that calendar.
    """
    calendar: TCCalendar = None
    calendarEvent: List[TCCalendarEvent] = ()


@dataclass
class CalendarUpdate(TcBaseObj):
    """
    The information necessary to modify a calendar.
    
    :var calendar: The calendar to modify
    :var calendarAttributes: The basic calendar data to modify
    :var eventsToAdd: New calendar events to add
    :var eventsToDelete: Existing calendar events to delete
    :var eventsToUpdate: Calendar events to update.
    """
    calendar: TCCalendar = None
    calendarAttributes: CalendarData = None
    eventsToAdd: List[CalendarEventData] = ()
    eventsToDelete: List[TCCalendarEvent] = ()
    eventsToUpdate: List[ModifyCalendarEvent] = ()


@dataclass
class CreateCalendarResponse(TcBaseObj):
    """
    The container with ALL the CalendarResponse and the service data.
    
    :var serviceData: The service data
    :var calendarResponse: The calendars and events
    """
    serviceData: ServiceData = None
    calendarResponse: List[CalendarResponse] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2009_10.DataManagement import CreateInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class OpenContextInfo(TcBaseObj):
    """
    Contains information on an opened context
    
    :var context: The opened context (The top line of the created window)
    :var object: The object which was passed to the method as input if this context was opened directly. If this
    context was opened as an outcome of another context or if CC was opened then this will be NULL
    :var views: A vector with the top lines of the opened views (OG windows)
    :var structureContext: The structure context containing this context. This is relevant only in case the context to
    open is a CC or SC object
    """
    context: BusinessObject = None
    object: BusinessObject = None
    views: List[BusinessObject] = ()
    structureContext: BusinessObject = None


@dataclass
class OpenContextInput(TcBaseObj):
    """
    Input for openContexts operation
    
    :var object: The object to open
    :var openViews: Defines whether to open all connected views
    :var openAssociatedContexts: Defines whether to open also associated contexts
    :var contextSettings: Defines additional context settings such as configuration (revision rule, variant rule, IC),
    show-unconfigured options, Product Configurator context flag, etc.
    """
    object: BusinessObject = None
    openViews: bool = False
    openAssociatedContexts: bool = False
    contextSettings: CreateInput = None


@dataclass
class OpenContextsResponse(TcBaseObj):
    """
    Response object for openContexts operation
    
    :var output: A vector with information of the method output
    :var serviceData: The operation ServiceData
    """
    output: List[ContextGroup] = ()
    serviceData: ServiceData = None


@dataclass
class OpenViewsResponse(TcBaseObj):
    """
    Response object for openViews operation
    
    :var views: A vector with the opened views (the top lines of the created OG windows)
    :var serviceData: The operation ServiceData
    """
    views: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class ContextGroup(TcBaseObj):
    """
    A group of contexts opened by openContexts operation
    
    :var contexts: A vector with information on each opened context in the group
    :var collaborationContext: The container of the context group (CC). This is relevant only in case the context to
    open is a CC object
    """
    contexts: List[OpenContextInfo] = ()
    collaborationContext: BusinessObject = None



from __future__ import annotations

from tcsoa.base import TcService


class ClassificationService(TcService):

    @classmethod
    def importClassificationDefinitions(cls, fileTicket: str, dryRun: bool) -> str:
        """
        Imports the json input containing classification definitions of type Cst0ClassDefinition,
        Cst0PropertyDefinition, Cst0KeyLOVDefinition or NodeDefinition and optionally validates them.
        The SOA accepts file ticket which will be used to read the json file contents from the transient volume, which
        will further be used to create classification definitions.
        The JSON file which is to be uploaded must follow one of the following JSON schema:-
        &bull;    TC_DATA\classification\json\1.1.0\schema\ Classification_Save_ClassDefinitions_Request.schema.json
        &bull;    TC_DATA\classification\json\1.1.0\schema\ Classification_Save_KeyLOVDefinitions_Request.schema.json
        &bull;    TC_DATA\classification\json\1.1.0\schema\ Classification_Save_PropertyDefinitions_Request.schema.json
        &bull;    TC_DATA\classification\json\1.0.0\schema\ Classification_Save_NodeDefinitions_Request.schema.json
        The response will include the information such as Name and IRDI (International Registration Data Identifier) of
        classification definitions.
        
        Use cases:
        This operation could be used when user wants to perform any of the following actions -
        &bull;    Validate json file without creating classification definitions.
        &bull;    Import a json file creating classification definitions.
        """
        return cls.execute_soa_method(
            method_name='importClassificationDefinitions',
            library='ClassificationCommon',
            service_date='2020_12',
            service_name='Classification',
            params={'fileTicket': fileTicket, 'dryRun': dryRun},
            response_cls=str,
        )

    @classmethod
    def searchClassificationDefinitions(cls, jsonRequest: str) -> str:
        """
        Searches classification definitions based on criteria provided in the input. Classification definitions include
        objects such as NodeDefinition, ClassDefinition, PropertyDefinition and KeyLOVDefinition. 
        The input criteria may include parameters that specify the property name, operator and the value to use while
        performing the search. 
        The response will include the number of objects found, the number of objects returned and object information
        such as Name and IRDI (International Registration Data Identifier).
        
        Use cases:
        This operation could be used when user wants to perform any of the following actions -
        &bull;    Get detailed information for specified object IRDI
        &bull;    Get objects matching the specified Name, Namespace, ID, Revision or any other object property.
        &bull;    Get objects sorted on specified property (eg. Last Modified Date)
        &bull;    Get top-level hierarchy of classes
        &bull;    Get paged search results
        """
        return cls.execute_soa_method(
            method_name='searchClassificationDefinitions',
            library='ClassificationCommon',
            service_date='2020_12',
            service_name='Classification',
            params={'jsonRequest': jsonRequest},
            response_cls=str,
        )
